@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco++/ncap2.cc,v 1.124 2012/01/01 20:51:54 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco++/ncap2.cc,v 1.172 2013/08/01 05:44:03 zender Exp $ */
 
 /* ncap2 -- netCDF arithmetic processor */
 
 /* Purpose: Compute user-defined derived fields using forward algebraic notation applied to netCDF files */
 
-/* Copyright (C) 1995--2012 Charlie Zender
+/* Copyright (C) 1995--2013 Charlie Zender
    
    License: GNU General Public License (GPL) Version 3
    The full license text is at http://www.gnu.org/copyleft/gpl.html 
@@ -20,7 +20,7 @@
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
    See the GNU General Public License for more details.
-   
+
    The original author of this software, Charlie Zender, seeks to improve
    it with your suggestions, contributions, bug-reports, and patches.
    Please contact the NCO project at http://nco.sf.net or write to
@@ -31,8 +31,8 @@
 
 /* Usage:
    ncap2 -O -D 1 -S ~/nco/data/ncap2.in ~/nco/data/in.nc ~/foo.nc
-   ncap2 -O -D 1 -v -S ~/nco/data/ncap2_tst.nco ~/nco/data/in.nc ~/foo.nc
-   ncap2 -O -D 1 -s two=one+two ~/nco/data/in.nc ~/foo.nc
+   ncap2 -O -D 1 -v -S ~/nco/data/ncap2_tst.nco  ~/nco/data/in.nc ~/foo.nc
+   ncap2 -O -D 1 -v -s two=one+two ~/nco/data/in.nc ~/foo.nc
    ncap2 -O -v -s 'foo=exp(0.61)' ~/nco/data/in.nc ~/foo.nc;ncks -C ~/foo.nc
    ncap2 -O -v -s 'defdim("dmn_tmp")=5;foo[$dmn_tmp]={0,2,4,6,8};foo2=one_dmn_rec_var(foo);print(foo);print(foo2);' ~/nco/data/in.nc ~/foo.nc;
    ncap2 -O -v -s 'foo=0*three_dmn_var_dbl;where(three_dmn_var_dbl>30){foo=three_dmn_var_dbl;}elsewhere{foo=three_dmn_var_dbl@_FillValue;};foo_avg=foo.avg($time);print(foo_avg);' ~/nco/data/in.nc ~/foo.nc
@@ -45,16 +45,19 @@
 
 // Standard C++ headers
 #include <string>
+#include <iterator> /* for inserter */
 
 // Standard C headers
 #include <assert.h>  /* assert() debugging macro */
 #include <math.h> /* sin cos cos sin 3.14159 */
 #include <stdio.h> /* stderr, FILE, NULL, etc. */
 #include <stdlib.h> /* atof, atoi, malloc, getopt */
-#include <string.h> /* strcmp. . . */
+#include <string.h> /* strcmp() */
 #include <sys/stat.h> /* stat() */
 #include <time.h> /* machine time */
-#include <unistd.h> /* all sorts of POSIX stuff */
+#ifndef _MSC_VER
+# include <unistd.h> /* POSIX stuff */
+#endif
 
 /* GNU getopt() is independent system header on FREEBSD, LINUX, LINUXALPHA, LINUXAMD, LINUXARM, WIN32
    AT&T getopt() is in unistd.h or stdlib.h on AIX, CRAY, NECSX, SUNMP, SUN4SOL2
@@ -73,8 +76,6 @@
 #include <gsl/gsl_rng.h>
 
 #endif // !ENABLE_GSL
-#include <netcdf.h> /* netCDF definitions and C library */
-#include "nco_netcdf.h"  /* NCO wrappers for libnetcdf.a */
 
 /* Personal headers */
 /* #define MAIN_PROGRAM_FILE MUST precede #include libnco.h */
@@ -98,28 +99,30 @@ int
 main(int argc,char **argv)
 {
   const char fnc_nm[]="main"; 
-  FILE *yyin; /* file handle used to check file existance */
+  FILE *yyin; /* File handle used to check file existance */
   int parse_antlr(std::vector<prs_cls> &prs_vtr ,char*,char*);
   
   /* fxm TODO nco652 */
   double rnd_nbr(double);
   
+  nco_bool ATT_INHERIT=True;          
+  nco_bool ATT_PROPAGATE=True;        
   nco_bool CNV_CCM_CCSM_CF;
-  nco_bool EXCLUDE_INPUT_LIST=False; /* Option c */
   nco_bool EXTRACT_ALL_COORDINATES=False; /* Option c */
   nco_bool EXTRACT_ASSOCIATED_COORDINATES=True; /* Option C */
-  nco_bool FILE_RETRIEVED_FROM_REMOTE_LOCATION;
+  nco_bool FL_RTR_RMT_LCN;
   nco_bool FL_LST_IN_FROM_STDIN=False; /* [flg] fl_lst_in comes from stdin */
   nco_bool FORCE_APPEND=False; /* Option A */
   nco_bool FORCE_OVERWRITE=False; /* Option O */
   nco_bool FORTRAN_IDX_CNV=False; /* Option F */
   nco_bool HISTORY_APPEND=True; /* Option h */
-  nco_bool OUTPUT_TO_NEW_NETCDF_FILE=False;
+  nco_bool FL_OUT_NEW=False;
   nco_bool PRN_FNC_TBL=False; /* Option f */  
   nco_bool PROCESS_ALL_VARS=True; /* Option v */  
-  nco_bool REMOVE_REMOTE_FILES_AFTER_PROCESSING=True; /* Option R */
-  nco_bool ATT_PROPAGATE=True;        
-  nco_bool ATT_INHERIT=True;          
+  nco_bool RAM_CREATE=False; /* [flg] Create file in RAM */
+  nco_bool RAM_OPEN=False; /* [flg] Open (netCDF3-only) file(s) in RAM */
+  nco_bool RM_RMT_FL_PST_PRC=True; /* Option R */
+  nco_bool WRT_TMP_FL=True; /* [flg] Write output to temporary file */
   nco_bool flg_cln=True; /* [flg] Clean memory prior to exit */
   
   char **fl_lst_abb=NULL_CEWI; /* Option n */
@@ -137,13 +140,13 @@ main(int argc,char **argv)
   char *fl_spt_usr=NULL_CEWI; /* Option s */
   char *lmt_arg[NC_MAX_DIMS];
   char *opt_crr=NULL_CEWI; /* [sng] String representation of current long-option name */
-  char *sng_cnv_rcd=char_CEWI; /* [sng] strtol()/strtoul() return code */
+  char *sng_cnv_rcd=NULL_CEWI; /* [sng] strtol()/strtoul() return code */
 #define NCAP_SPT_NBR_MAX 100
   char *spt_arg[NCAP_SPT_NBR_MAX]; /* fxm: Arbitrary size, should be dynamic */
   char *spt_arg_cat=NULL_CEWI; /* [sng] User-specified script */
   
-  const char * const CVS_Id="$Id: ncap2.cc,v 1.124 2012/01/01 20:51:54 zender Exp $"; 
-  const char * const CVS_Revision="$Revision: 1.124 $";
+  const char * const CVS_Id="$Id: ncap2.cc,v 1.172 2013/08/01 05:44:03 zender Exp $"; 
+  const char * const CVS_Revision="$Revision: 1.172 $";
   const char * const att_nm_tmp="eulaVlliF_"; /* For netCDF4 name hack */
   const char * const opt_sht_lst="346ACcD:FfhL:l:n:Oo:p:Rrs:S:t:vx-:"; /* [sng] Single letter command line options */
   
@@ -152,8 +155,7 @@ main(int argc,char **argv)
   dmn_sct **dmn_in=NULL_CEWI;  /* [lst] Dimensions in input file */
   dmn_sct *dmn_new;
   dmn_sct *dmn_item;
-  
-  // Template lists
+    // Template lists
   NcapVector<dmn_sct*> dmn_in_vtr;  
   NcapVector<dmn_sct*> dmn_out_vtr;  
   
@@ -185,6 +187,7 @@ main(int argc,char **argv)
   int idx;
   int jdx;
   int lmt_nbr=0; /* Option d. NB: lmt_nbr gets incremented */
+  int md_open; /* [enm] Mode flag for nc_open() call */
   int nbr_dmn_ass=int_CEWI;/* Number of dimensions in temporary list */
   int nbr_dmn_in=int_CEWI; /* Number of dimensions in dim_in */
   int nbr_lst_a=0; /* size of xtr_lst_a */
@@ -192,7 +195,7 @@ main(int argc,char **argv)
   int nbr_var_fix; /* nbr_var_fix gets incremented */
   int nbr_var_fl;/* number of vars in a file */
   int nbr_var_prc; /* nbr_var_prc gets incremented */
-  int nbr_xtr=0; /* nbr_xtr will not otherwise be set for -c with no -v */
+  int xtr_nbr=0; /* xtr_nbr will not otherwise be set for -c with no -v */
   int opt;
   int out_id;  
   int rcd=NC_NOERR; /* [rcd] Return code */
@@ -205,7 +208,9 @@ main(int argc,char **argv)
   nm_id_sct *xtr_lst=NULL_CEWI; /* Non-processed variables to copy to OUTPUT */
   nm_id_sct *xtr_lst_a=NULL_CEWI; /* Initialize to ALL variables in OUTPUT file */
   
+  size_t bfr_sz_hnt=NC_SIZEHINT_DEFAULT; /* [B] Buffer size hint */
   size_t cnk_sz_scl=0UL; /* [nbr] Chunk size scalar */
+  size_t hdr_pad=0UL; /* [B] Pad at end of header section */
   size_t sng_lng;
   size_t spt_arg_lng=size_t_CEWI;
 
@@ -227,10 +232,20 @@ main(int argc,char **argv)
       {"drt",no_argument,0,0}, /* [flg] Allow dirty memory on exit */
       {"dirty",no_argument,0,0}, /* [flg] Allow dirty memory on exit */
       {"mmr_drt",no_argument,0,0}, /* [flg] Allow dirty memory on exit */
+      {"hdf_upk",no_argument,0,0}, /* [flg] HDF unpack convention: unpacked=scale_factor*(packed-add_offset) */
+      {"hdf_unpack",no_argument,0,0}, /* [flg] HDF unpack convention: unpacked=scale_factor*(packed-add_offset) */
+      {"ram_all",no_argument,0,0}, /* [flg] Open (netCDF3) and create file(s) in RAM */
+      {"create_ram",no_argument,0,0}, /* [flg] Create file in RAM */
+      {"open_ram",no_argument,0,0}, /* [flg] Open (netCDF3) file(s) in RAM */
+      {"diskless_all",no_argument,0,0}, /* [flg] Open (netCDF3) and create file(s) in RAM */
+      {"wrt_tmp_fl",no_argument,0,0}, /* [flg] Write output to temporary file */
+      {"write_tmp_fl",no_argument,0,0}, /* [flg] Write output to temporary file */
+      {"no_tmp_fl",no_argument,0,0}, /* [flg] Do not write output to temporary file */
       {"version",no_argument,0,0},
       {"vrs",no_argument,0,0},
       /* Long options with argument, no short option counterpart */
-      {"cnk_map",required_argument,0,0}, /* [nbr] Chunking map */
+      {"bfr_sz_hnt",required_argument,0,0}, /* [B] Buffer size hint */
+      {"buffer_size_hint",required_argument,0,0}, /* [B] Buffer size hint */
       {"chunk_map",required_argument,0,0}, /* [nbr] Chunking map */
       {"cnk_plc",required_argument,0,0}, /* [nbr] Chunking policy */
       {"chunk_policy",required_argument,0,0}, /* [nbr] Chunking policy */
@@ -238,7 +253,9 @@ main(int argc,char **argv)
       {"chunk_scalar",required_argument,0,0}, /* [nbr] Chunk size scalar */
       {"cnk_dmn",required_argument,0,0}, /* [nbr] Chunk size */
       {"chunk_dimension",required_argument,0,0}, /* [nbr] Chunk size */
-      {"file_format",required_argument,0,0},
+      {"fl_fmt",required_argument,0,0},
+      {"hdr_pad",required_argument,0,0},
+      {"header_pad",required_argument,0,0},
       /* Long options with short counterparts */
       {"3",no_argument,0,'3'},
       {"4",no_argument,0,'4'},
@@ -283,6 +300,7 @@ main(int argc,char **argv)
       {"exclude",no_argument,0,'x'},
       {"xcl",no_argument,0,'x'},
       {"help",no_argument,0,'?'},
+      {"hlp",no_argument,0,'?'},
       {0,0,0,0}
     }; /* end opt_lng */
   int opt_idx=0; /* Index of current long option into opt_lng array */
@@ -303,6 +321,10 @@ main(int argc,char **argv)
     
     /* Process long options without short option counterparts */
     if(opt == 0){
+      if(!strcmp(opt_crr,"bfr_sz_hnt") || !strcmp(opt_crr,"buffer_size_hint")){
+	bfr_sz_hnt=strtoul(optarg,&sng_cnv_rcd,NCO_SNG_CNV_BASE10);
+	if(*sng_cnv_rcd) nco_sng_cnv_err(optarg,"strtoul",sng_cnv_rcd);
+      } /* endif cnk */
       if(!strcmp(opt_crr,"cnk_dmn") || !strcmp(opt_crr,"chunk_dimension")){
 	/* Copy limit argument for later processing */
 	cnk_arg[cnk_nbr]=(char *)strdup(optarg);
@@ -325,10 +347,19 @@ main(int argc,char **argv)
       if(!strcmp(opt_crr,"cln") || !strcmp(opt_crr,"mmr_cln") || !strcmp(opt_crr,"clean")) flg_cln=True; /* [flg] Clean memory prior to exit */
       if(!strcmp(opt_crr,"drt") || !strcmp(opt_crr,"mmr_drt") || !strcmp(opt_crr,"dirty")) flg_cln=False; /* [flg] Clean memory prior to exit */
       if(!strcmp(opt_crr,"fl_fmt") || !strcmp(opt_crr,"file_format")) rcd=nco_create_mode_prs(optarg,&fl_out_fmt);
+      if(!strcmp(opt_crr,"hdr_pad") || !strcmp(opt_crr,"header_pad")){
+        hdr_pad=strtoul(optarg,&sng_cnv_rcd,NCO_SNG_CNV_BASE10);
+        if(*sng_cnv_rcd) nco_sng_cnv_err(optarg,"strtoul",sng_cnv_rcd);
+      } /* endif "hdr_pad" */
+      if(!strcmp(opt_crr,"hdf_upk") || !strcmp(opt_crr,"hdf_unpack")) nco_upk_cnv=nco_upk_HDF; /* [flg] HDF unpack convention: unpacked=scale_factor*(packed-add_offset) */
+      if(!strcmp(opt_crr,"ram_all") || !strcmp(opt_crr,"create_ram") || !strcmp(opt_crr,"diskless_all")) RAM_CREATE=True; /* [flg] Open (netCDF3) file(s) in RAM */
+      if(!strcmp(opt_crr,"ram_all") || !strcmp(opt_crr,"open_ram") || !strcmp(opt_crr,"diskless_all")) RAM_OPEN=True; /* [flg] Create file in RAM */
       if(!strcmp(opt_crr,"vrs") || !strcmp(opt_crr,"version")){
 	(void)nco_vrs_prn(CVS_Id,CVS_Revision);
 	nco_exit(EXIT_SUCCESS);
       } /* endif "vrs" */
+      if(!strcmp(opt_crr,"wrt_tmp_fl") || !strcmp(opt_crr,"write_tmp_fl")) WRT_TMP_FL=True;
+      if(!strcmp(opt_crr,"no_tmp_fl")) WRT_TMP_FL=False;
     } /* opt != 0 */
     /* Process short options */
     switch(opt){
@@ -389,7 +420,7 @@ main(int argc,char **argv)
       fl_pth=(char *)strdup(optarg);
       break;
     case 'R': /* Toggle removal of remotely-retrieved-files. Default is True. */
-      REMOVE_REMOTE_FILES_AFTER_PROCESSING=!REMOVE_REMOTE_FILES_AFTER_PROCESSING;
+      RM_RMT_FL_PST_PRC=!RM_RMT_FL_PST_PRC;
       break;
     case 'r': /* Print CVS program information and copyright notice */
       (void)nco_vrs_prn(CVS_Id,CVS_Revision);
@@ -411,10 +442,7 @@ main(int argc,char **argv)
       break;
     case 'v': /* Variables to extract/exclude */
       PROCESS_ALL_VARS=False;
-      nbr_xtr=0;
-      break;
-    case 'x': /* Exclude rather than extract variables specified with -v */
-      EXCLUDE_INPUT_LIST=True;
+      xtr_nbr=0;
       break;
     case '?': /* Print proper usage */
       (void)nco_usg_prn();
@@ -424,6 +452,7 @@ main(int argc,char **argv)
       err_prn(fnc_nm,"Long options are not available in this build. Use single letter options instead.\n");
       break;
     default: /* Print proper usage */
+      (void)fprintf(stdout,"%s ERROR in command-line syntax/options. Please reformulate command accordingly.\n",prg_nm_get());
       (void)nco_usg_prn();
       nco_exit(EXIT_FAILURE);
       break;
@@ -475,6 +504,11 @@ main(int argc,char **argv)
   arr_cls arr_obj(true); 
   // Bilinear interpolation functions
   bil_cls bil_obj(true); 
+  // Co-ordinates functions
+  cod_cls cod_obj(true); 
+
+  // misc functions 
+  misc_cls misc_obj(true);
 
   // Populate vector
   (void)pop_fmc_vtr(fmc_vtr,&cnv_obj);
@@ -490,6 +524,16 @@ main(int argc,char **argv)
   (void)pop_fmc_vtr(fmc_vtr,&unr_obj);
   (void)pop_fmc_vtr(fmc_vtr,&arr_obj);
   (void)pop_fmc_vtr(fmc_vtr,&bil_obj);
+  (void)pop_fmc_vtr(fmc_vtr,&cod_obj);
+  (void)pop_fmc_vtr(fmc_vtr,&misc_obj);
+
+#ifdef ENABLE_NCO_GSL
+  // nco_gsl functions
+  nco_gsl_cls nco_gsl_obj(true); 
+
+  // Populate vector
+  (void)pop_fmc_vtr(fmc_vtr,&nco_gsl_obj);
+#endif //ENABLE_NCO_GSL
    
   // GSL functions
 #ifdef ENABLE_GSL
@@ -515,14 +559,11 @@ main(int argc,char **argv)
   
   if(ncap_gsl_mode_prec<0 || ncap_gsl_mode_prec>2) ncap_gsl_mode_prec=0;
 
-  /* create a generator chosen by the 
-    environment variables GSL_RNG_TYPE ,GSL_RNG_SEED */
-     
+  /* create generator chosen by environment variables GSL_RNG_TYPE, GSL_RNG_SEED */
   gsl_rng_env_setup();
-
 #endif // !ENABLE_GSL
   
-  //Sort Vector 
+  // Sort Vector 
   std::sort(fmc_vtr.begin(),fmc_vtr.end());
   
   if(PRN_FNC_TBL){
@@ -537,7 +578,7 @@ main(int argc,char **argv)
   
   /* Process positional arguments and fill in filenames */
   fl_lst_in=nco_fl_lst_mk(argv,argc,optind,&fl_nbr,&fl_out,&FL_LST_IN_FROM_STDIN);
-  if(fl_out) OUTPUT_TO_NEW_NETCDF_FILE=True; else fl_out=(char *)strdup(fl_lst_in[0]);
+  if(fl_out) FL_OUT_NEW=True; else fl_out=(char *)strdup(fl_lst_in[0]);
   
   /* Make uniform list of user-specified chunksizes */
   if(cnk_nbr > 0) cnk=nco_cnk_prs(cnk_nbr,cnk_arg);
@@ -548,9 +589,10 @@ main(int argc,char **argv)
   /* Parse filename */
   fl_in=nco_fl_nm_prs(fl_in,0,&fl_nbr,fl_lst_in,abb_arg_nbr,fl_lst_abb,fl_pth);
   /* Make sure file is on local system and is readable or die trying */
-  fl_in=nco_fl_mk_lcl(fl_in,fl_pth_lcl,&FILE_RETRIEVED_FROM_REMOTE_LOCATION);
-  /* Open file for reading */
-  rcd=nco_open(fl_in,NC_NOWRITE,&in_id);
+  fl_in=nco_fl_mk_lcl(fl_in,fl_pth_lcl,&FL_RTR_RMT_LCN);
+  /* Open file using appropriate buffer size hints and verbosity */
+  if(RAM_OPEN) md_open=NC_NOWRITE|NC_DISKLESS; else md_open=NC_NOWRITE;
+  rcd+=nco_fl_open(fl_in,md_open,&bfr_sz_hnt,&in_id);
   (void)nco_inq_format(in_id,&fl_in_fmt);
   
   /* Form list of all dimensions in file */  
@@ -559,7 +601,7 @@ main(int argc,char **argv)
   //dmn_in=(dmn_sct **)nco_malloc(nbr_dmn_in*sizeof(dmn_sct *));
   for(idx=0;idx<nbr_dmn_in;idx++) dmn_in_vtr.push_back(nco_dmn_fll(in_id,dmn_lst[idx].id,dmn_lst[idx].nm));
   
-  //sort vector
+  // Sort vector
   dmn_in_vtr.sort();
   dmn_in=&dmn_in_vtr[0];
   
@@ -573,14 +615,15 @@ main(int argc,char **argv)
   (void)nco_fl_fmt_vet(fl_out_fmt,cnk_nbr,dfl_lvl);
 
   /* Open output file */
-  if(OUTPUT_TO_NEW_NETCDF_FILE){
+  if(FL_OUT_NEW){
     /* Normal case, like rest of NCO, where writes are made to temporary file */
-    fl_out_tmp=nco_fl_out_open(fl_out,FORCE_APPEND,FORCE_OVERWRITE,fl_out_fmt,&out_id);
+    fl_out_tmp=nco_fl_out_open(fl_out,FORCE_APPEND,FORCE_OVERWRITE,fl_out_fmt,&bfr_sz_hnt,RAM_CREATE,RAM_OPEN,WRT_TMP_FL,&out_id);
   }else{ /* Existing file */
     /* ncap2, like ncrename and ncatted, directly modifies fl_in if fl_out is omitted
        If fl_out resolves to _same name_ as fl_in, method above is employed */
     fl_out_tmp=(char *)strdup(fl_out);
-    rcd=nco_open(fl_out_tmp,NC_WRITE,&out_id);
+    if(RAM_OPEN) md_open=NC_WRITE|NC_DISKLESS; else md_open=NC_WRITE;
+    rcd+=nco_fl_open(fl_out_tmp,md_open,&bfr_sz_hnt,&out_id);
     (void)nco_redef(out_id);
   } /* Existing file */
   
@@ -591,9 +634,15 @@ main(int argc,char **argv)
   if(HISTORY_APPEND) (void)nco_hst_att_cat(out_id,cmd_ln);
   
   if(thr_nbr > 0 && HISTORY_APPEND) (void)nco_thr_att_cat(out_id,thr_nbr);
-  
-  (void)nco_enddef(out_id);
-  
+
+  /* Take output file out of define mode */
+  if(hdr_pad == 0UL){
+    (void)nco_enddef(out_id);
+  }else{
+    (void)nco__enddef(out_id,hdr_pad);
+    if(dbg_lvl >= nco_dbg_scl) (void)fprintf(stderr,"%s: INFO Padding header with %lu extra bytes\n",prg_nm_get(),(unsigned long)hdr_pad);
+  } /* hdr_pad */
+    
   /* Set arguments for script execution NB: all these are used as references */
   prs_cls prs_arg(dmn_in_vtr,dmn_out_vtr,fmc_vtr,var_vtr,int_vtr);
   
@@ -601,20 +650,20 @@ main(int argc,char **argv)
   prs_arg.in_id=in_id; /* [id] Input data file ID */
   prs_arg.fl_out=fl_out; /* [sng] Output data file */
   prs_arg.out_id=out_id; /* [id] Output data file ID */
-  //rcd=nco_open(fl_out_tmp, NC_NOWRITE|NC_SHARE,&prs_arg.out_id_readonly); /* Read Output file */
-  rcd=nco_open(fl_out_tmp,NC_NOWRITE,&prs_arg.out_id_readonly); /* Read Output file */
+  if(RAM_OPEN) md_open=NC_NOWRITE|NC_DISKLESS; else md_open=NC_NOWRITE;
+  rcd+=nco_fl_open(fl_out_tmp,md_open,&bfr_sz_hnt,&prs_arg.out_id_readonly);
   
   prs_arg.FORTRAN_IDX_CNV=FORTRAN_IDX_CNV;
   prs_arg.ATT_PROPAGATE=ATT_PROPAGATE;      
   prs_arg.ATT_INHERIT=ATT_INHERIT;
-  prs_arg.NCAP_MPI_SORT=(thr_nbr>1 ? true:false);
+  prs_arg.NCAP_MPI_SORT=(thr_nbr > 1 ? true:false);
   
   prs_arg.dfl_lvl=dfl_lvl;  /* [enm] Deflate level */
   prs_arg.cnk_sz=(size_t*)NULL; /* Chunk sizes NULL for now */ 
   
 #ifdef NCO_NETCDF4_AND_FILLVALUE
   prs_arg.NCAP4_FILL = (fl_out_fmt==NC_FORMAT_NETCDF4 || fl_out_fmt==NC_FORMAT_NETCDF4_CLASSIC);
-# else
+#else
   prs_arg.NCAP4_FILL=false;
 #endif
   prs_arg.ntl_scn=false;
@@ -625,30 +674,77 @@ main(int argc,char **argv)
   for(idx=1;idx<thr_nbr;idx++){
     prs_cls prs_tmp(prs_arg);
     
-    // Open files for each thread
-    rcd=nco_open(fl_in,NC_NOWRITE,&prs_tmp.in_id);
+    /* Open files for each thread */
+    if(RAM_OPEN) md_open=NC_NOWRITE|NC_DISKLESS; else md_open=NC_NOWRITE;
+    rcd+=nco_fl_open(fl_in,md_open,&bfr_sz_hnt,&prs_tmp.in_id);
     
-    // Handle to read output only
-    //rcd=nco_open(fl_out_tmp, NC_NOWRITE|NC_SHARE,&prs_tmp.out_id_readonly);
-    rcd=nco_open(fl_out_tmp, NC_NOWRITE,&prs_tmp.out_id_readonly);
+    /* Handle to read output only */
+    rcd+=nco_fl_open(fl_out_tmp,md_open,&bfr_sz_hnt,&prs_tmp.out_id_readonly);
     
-    // only one handle for reading & writing 
+    /* Only one handle for reading and writing */
     prs_tmp.out_id=out_id;
     
     prs_vtr.push_back(prs_tmp);
   } /* end loop over threads */
+
+  /* If we are appending (-A) then output file already contains dimensions/variables */
+  if(FORCE_APPEND){
+    int nbr_dmn_out=0;  
+    int nbr_var_fl=0;
+    int nbr_xtr=0;
+    nm_id_sct *dmn_lst=NULL_CEWI;
+    nm_id_sct *xtr_lst=NULL_CEWI;
+    NcapVar *Nvar;
+
+    var_sct *var_tmp;
+
+    /* Form list of all dimensions in file */  
+    dmn_lst=nco_dmn_lst(out_id,&nbr_dmn_out);
   
+    //dmn_in=(dmn_sct **)nco_malloc(nbr_dmn_in*sizeof(dmn_sct *));
+    for(idx=0;idx<nbr_dmn_out;idx++) dmn_out_vtr.push_back(nco_dmn_fll(out_id,dmn_lst[idx].id,dmn_lst[idx].nm));
+
+    // Sort vector
+    dmn_out_vtr.sort();
+
+    /* xrf dmn_out with dmn_in_vtr */
+    for(idx=0;idx<nbr_dmn_out;idx++){
+      dmn_item=dmn_in_vtr.find(dmn_out_vtr[idx]->nm);
+      if(!dmn_item) continue; 
+      (void)nco_dmn_xrf(dmn_item,dmn_out_vtr[idx]);
+      if(dmn_item->sz != dmn_out_vtr[idx]->sz) 
+        (void)fprintf(stdout,"%s: WARNING: dimension miss-match. Dimension \"%s\" is size=%ld in input file and size=%ld in output file.\nHINT: Command may work if Append mode (-A) is not used.\n",prg_nm_get(),dmn_item->nm,dmn_item->sz,dmn_out_vtr[idx]->sz);
+    } /* end loop over dimensions */
+
+    /* Get number of variables in output file */
+    rcd=nco_inq(out_id,(int *)NULL,&nbr_var_fl,(int *)NULL,(int *)NULL);
+  
+    /* Make list of all new variables in output_file */  
+    xtr_lst=nco_var_lst_mk(out_id,nbr_var_fl,(char**)NULL,False,False,&nbr_xtr);      
+    for(idx=0;idx<nbr_var_fl;idx++){
+      var_tmp=prs_arg.ncap_var_init(xtr_lst[idx].nm,false);
+      Nvar=new NcapVar(var_tmp); 
+      Nvar->flg_stt=2;   
+      prs_arg.var_vtr.push(Nvar);
+      /* Copy output attributes into var_vtr */
+      ncap_att_gnrl(xtr_lst[idx].nm, xtr_lst[idx].nm,2,&prs_arg);        
+    } /* end loop over variables */
+
+    /* Free lists */
+    dmn_lst=nco_nm_id_lst_free(dmn_lst,nbr_dmn_out);
+    xtr_lst=nco_nm_id_lst_free(xtr_lst,nbr_xtr);
+  } /* !FORCE_APPEND */
+
   if(fl_spt_usr == NULL_CEWI){
     /* No script file specified, look for command-line scripts */
-    if(nbr_spt == 0)
-      err_prn(fnc_nm,"No script file or command line scripts specified\nHINT Use, e.g., -s \"foo=bar\"\n");
-    
+    if(nbr_spt == 0) err_prn(fnc_nm,"No script file or command line scripts specified\nHINT Use, e.g., -s \"foo=bar\"\n");
+
     /* Print all command-line scripts */
-    if(dbg_lvl_get() > 0){
+    if(dbg_lvl_get() >= nco_dbg_std){
       for(idx=0;idx<nbr_spt;idx++) 
-	(void)fprintf(stderr,"spt_arg[%d] = %s\n",idx,spt_arg[idx]);
+        (void)fprintf(stderr,"spt_arg[%d] = %s\n",idx,spt_arg[idx]);
     } /* endif debug */
-    
+
     /* Parse command line scripts */
     fl_spt_usr=(char *)strdup("Command-line script");
   }else{ /* ...endif command-line scripts, begin script file... */
@@ -662,39 +758,24 @@ main(int argc,char **argv)
   rcd=parse_antlr(prs_vtr,fl_spt_usr,spt_arg_cat);
   
   /* Get number of variables in output file */
-  rcd=nco_inq(out_id,(int *)NULL,&nbr_var_fl,(int *)NULL,(int*)NULL);
+  rcd=nco_inq(out_id,(int *)NULL,&nbr_var_fl,(int *)NULL,(int *)NULL);
   
   /* Make list of all new variables in output_file */  
   xtr_lst_a=nco_var_lst_mk(out_id,nbr_var_fl,var_lst_in,False,False,&nbr_lst_a);
+
   
   if(PROCESS_ALL_VARS){
     /* Get number of variables in input file */
     rcd=nco_inq(in_id,(int *)NULL,&nbr_var_fl,(int *)NULL,(int *)NULL);
-    
     /* Form initial list of all variables in input file */
-    xtr_lst=nco_var_lst_mk(in_id,nbr_var_fl,var_lst_in,False,False,&nbr_xtr);
+    xtr_lst=nco_var_lst_mk(in_id,nbr_var_fl,var_lst_in,False,False,&xtr_nbr);
   }else{
     /* Make list of variables of new attributes whose parent variable is only in input file */
-    xtr_lst=nco_att_lst_mk(in_id,out_id,var_vtr,&nbr_xtr);
+    xtr_lst=nco_att_lst_mk(in_id,out_id,var_vtr,&xtr_nbr);
   } /* endif */
-  
-    /* Find dimensions associated with xtr_lst */
-    /* Write to O only new dims
-       Add apropriate coordinate variables to extraction list 
-       options -c      -process all cordinates 
-       i.e., add coordinates to var list 
-       Also add their dims
-       
-       options --none   -process associated co-ords
-       loop though dim_out and append to var list
-       
-       options -C         no co-ordinates   Do nothing */
-  
-    /* Subtract list A again */
-    /* Finally extract variables on list */
-  
+
   /* Subtract list A */
-  if(nbr_lst_a > 0) xtr_lst=nco_var_lst_sub(xtr_lst,&nbr_xtr,xtr_lst_a,nbr_lst_a);
+  if(nbr_lst_a > 0) xtr_lst=nco_var_lst_sub(xtr_lst,&xtr_nbr,xtr_lst_a,nbr_lst_a);
   
   /* Put file in define mode to allow metadata writing */
   (void)nco_redef(out_id);
@@ -703,19 +784,17 @@ main(int argc,char **argv)
   dmn_lst=nco_nm_id_lst_free(dmn_lst,nbr_dmn_in);
   
   /* Make list of dimensions of variables in xtr_lst */
-  if(nbr_xtr > 0) dmn_lst=nco_dmn_lst_ass_var(in_id,xtr_lst,nbr_xtr,&nbr_dmn_ass);
+  if(xtr_nbr > 0) dmn_lst=nco_dmn_lst_ass_var(in_id,xtr_lst,xtr_nbr,&nbr_dmn_ass);
   
   /* Find and add any new dimensions to output */
   for(idx=0;idx<nbr_dmn_ass;idx++){
-    dmn_item=dmn_out_vtr.find(dmn_lst[idx].nm);
-    if(dmn_item) continue;    
-    dmn_item=dmn_in_vtr.find(dmn_lst[idx].nm);
-    if(dmn_item == NULL) continue;
-    dmn_new=nco_dmn_dpl(dmn_item);
-    (void)nco_dmn_dfn(fl_out,out_id,&dmn_new,1);
-    (void)nco_dmn_xrf(dmn_new,dmn_item);
-    (void)dmn_out_vtr.push_back(dmn_new);
-  }  /* end loop over idx */
+    jdx=dmn_in_vtr.findis(dmn_lst[idx].nm);
+    if(dmn_in_vtr[jdx]->xrf ) continue;
+
+    (void)dmn_out_vtr.push_back(nco_dmn_dpl(dmn_in_vtr[jdx]));
+    (void)nco_dmn_dfn(fl_out,out_id,&dmn_out_vtr.back(),1);
+    (void)nco_dmn_xrf(dmn_out_vtr.back(),dmn_in_vtr[jdx]);
+  }  
   
   /* Free current list of all dimensions in input file */
   dmn_lst=nco_nm_id_lst_free(dmn_lst,nbr_dmn_ass);
@@ -723,116 +802,156 @@ main(int argc,char **argv)
   /* Dimensions for manually specified extracted variables are now defined in output file
      Add coordinate variables to extraction list
      If EXTRACT_ALL_COORDINATES then write associated dimension to output */
-  
   if(EXTRACT_ASSOCIATED_COORDINATES){
     for(idx=0;idx<dmn_in_vtr.size();idx++){
-      if(!dmn_in_vtr[idx]->is_crd_dmn) continue;
-      
-      if(EXTRACT_ALL_COORDINATES && !dmn_in_vtr[idx]->xrf){
-	/* Add dimensions to output list dmn_out */
-	dmn_item=dmn_in_vtr[idx];
-	dmn_new=nco_dmn_dpl(dmn_item);
-	(void)nco_dmn_xrf(dmn_new,dmn_item);
-	/* Write dimension to output */
-	(void)nco_dmn_dfn(fl_out,out_id,&dmn_new,1);
-	(void)dmn_out_vtr.push_back(dmn_new);
+      dmn_item=dmn_in_vtr[idx]; /*de-reference */
+      if(!dmn_item->is_crd_dmn) continue;
+
+      if(EXTRACT_ALL_COORDINATES && !dmn_item->xrf){
+        /* Add dimensions to output list dmn_out */
+        dmn_new=nco_dmn_dpl(dmn_item);
+        (void)nco_dmn_xrf(dmn_new,dmn_item);
+        /* Write dimension to output */
+        (void)nco_dmn_dfn(fl_out,out_id,&dmn_new,1);
+        (void)dmn_out_vtr.push_back(dmn_new);
       } /* end if */
       /* Add coordinate variable to extraction list, dimension has already been output */
-      if(dmn_in_vtr[idx]->xrf){
-	for(jdx=0;jdx<nbr_xtr;jdx++)
-	  if(!strcmp(xtr_lst[jdx].nm,dmn_in_vtr[idx]->nm)) break;
-	
-	if(jdx != nbr_xtr) continue;
-	/* If coordinate is not on list then add it to extraction list */
-	xtr_lst=(nm_id_sct *)nco_realloc(xtr_lst,(nbr_xtr+1)*sizeof(nm_id_sct));
-	xtr_lst[nbr_xtr].nm=(char *)strdup(dmn_in_vtr[idx]->nm);
-	xtr_lst[nbr_xtr++].id=dmn_in_vtr[idx]->cid;
+      if(dmn_item->xrf){
+        for(jdx=0;jdx<xtr_nbr;jdx++)
+          if(!strcmp(xtr_lst[jdx].nm,dmn_item->nm)) break;
+
+        if(jdx != xtr_nbr) continue;
+        /* If coordinate is not on list then add it to extraction list */
+        xtr_lst=(nm_id_sct *)nco_realloc(xtr_lst,(xtr_nbr+1)*sizeof(nm_id_sct));
+        xtr_lst[xtr_nbr].nm=(char *)strdup(dmn_item->nm);
+        xtr_lst[xtr_nbr].id=dmn_item->cid;
+
+        /* Increment */
+        xtr_nbr++;
+
       } /* endif */
     } /* end loop over idx */	      
   } /* end if */ 
-  
+
   /* Is this a CCM/CCSM/CF-format history tape? */
   CNV_CCM_CCSM_CF=nco_cnv_ccm_ccsm_cf_inq(in_id);
-  
+
   /* Add coordinates defined by CF convention */
-  if(CNV_CCM_CCSM_CF && (EXTRACT_ALL_COORDINATES || EXTRACT_ASSOCIATED_COORDINATES)) xtr_lst=nco_cnv_cf_crd_add(in_id,xtr_lst,&nbr_xtr);
-  
+  if(CNV_CCM_CCSM_CF && (EXTRACT_ALL_COORDINATES || EXTRACT_ASSOCIATED_COORDINATES)) xtr_lst=nco_cnv_cf_crd_add(in_id,xtr_lst,&xtr_nbr);
+
   /* Subtract list A again (it may contain re-defined coordinates) */
-  if(nbr_xtr > 0) xtr_lst=nco_var_lst_sub(xtr_lst,&nbr_xtr,xtr_lst_a,nbr_lst_a);
-  
+  if(xtr_nbr > 0) xtr_lst=nco_var_lst_sub(xtr_lst,&xtr_nbr,xtr_lst_a,nbr_lst_a);
+
   /* Sort extraction list for faster I/O */
-  if(nbr_xtr > 1) xtr_lst=nco_lst_srt_nm_id(xtr_lst,nbr_xtr,False);
-  
+  if(xtr_nbr > 1) xtr_lst=nco_lst_srt_nm_id(xtr_lst,xtr_nbr,False);
+
   /* Write "fixed" variables */
-  var=(var_sct **)nco_malloc(nbr_xtr*sizeof(var_sct *));
-  var_out=(var_sct **)nco_malloc(nbr_xtr*sizeof(var_sct *));
-  for(idx=0;idx<nbr_xtr;idx++){
+  var=(var_sct **)nco_malloc(xtr_nbr*sizeof(var_sct *));
+  var_out=(var_sct **)nco_malloc(xtr_nbr*sizeof(var_sct *));
+  for(idx=0;idx<xtr_nbr;idx++){
     var[idx]=nco_var_fll(in_id,xtr_lst[idx].id,xtr_lst[idx].nm,&dmn_in_vtr[0],dmn_in_vtr.size());
     var_out[idx]=nco_var_dpl(var[idx]);
     (void)nco_xrf_var(var[idx],var_out[idx]);
     (void)nco_xrf_dmn(var_out[idx]);
   } /* end loop over idx */
-  
+
   /* NB: ncap is not well-suited for nco_var_lst_dvd() */
   /* Divide variable lists into lists of fixed variables and variables to be processed */
-  (void)nco_var_lst_dvd(var,var_out,nbr_xtr,CNV_CCM_CCSM_CF,nco_pck_plc_nil,nco_pck_map_nil,(dmn_sct **)NULL,(int)0,&var_fix,&var_fix_out,&nbr_var_fix,&var_prc,&var_prc_out,&nbr_var_prc);
+  (void)nco_var_lst_dvd(var,var_out,xtr_nbr,CNV_CCM_CCSM_CF,True,nco_pck_plc_nil,nco_pck_map_nil,(dmn_sct **)NULL,(int)0,&var_fix,&var_fix_out,&nbr_var_fix,&var_prc,&var_prc_out,&nbr_var_prc);
   
   /* csz: Why not call this with var_fix? */
   /* Define non-processed vars */
-  (void)nco_var_dfn(in_id,fl_out,out_id,var_out,nbr_xtr,(dmn_sct **)NULL,(int)0,nco_pck_plc_nil,nco_pck_map_nil,dfl_lvl);
+  (void)nco_var_dfn(in_id,fl_out,out_id,var_out,xtr_nbr,(dmn_sct **)NULL,(int)0,nco_pck_plc_nil,nco_pck_map_nil,dfl_lvl);
   
   /* Write out new attributes possibly overwriting old ones */
   for(idx=0;idx<var_vtr.size();idx++){
-    /* write misssing value contained inside var */
+
+   
     if(var_vtr[idx]->xpr_typ == ncap_var){
+      int rcd_inq_att;
       var_sct *var_ref;
       
       var_ref=var_vtr[idx]->var;
       rcd=nco_inq_varid_flg(out_id,var_ref->nm,&var_id);
       
-      /* skip RAM vars and orphaned vars */
-      if(rcd!=NC_NOERR|| !var_ref->has_mss_val) continue;  
+      /* filter out ram vars */ 
+      if(rcd!=NC_NOERR || !var_ref->has_mss_val) continue;  
       
-      /* Do netdf4 name hack */
-      if(prs_arg.NCAP4_FILL){
+      /* see if missing value already present */
+      rcd_inq_att=nco_inq_att_flg(out_id,var_id,nco_mss_val_sng_get(),&att_item.type,&att_item.sz);
+
+      /* type mismatch dont overwrite. This can occur when packed vars are in output from using the -A switch */
+      if(rcd_inq_att==NC_NOERR && var_ref->type != att_item.type) continue;   
+
+      /* fill mode and att exists */ 
+      if(prs_arg.NCAP4_FILL && rcd_inq_att==NC_NOERR){
+        (void)nco_rename_att(out_id,var_id,nco_mss_val_sng_get(),att_nm_tmp);     
+        (void)nco_put_att(out_id,var_id,att_nm_tmp,var_ref->type,1,var_ref->mss_val.vp);   
+        (void)nco_rename_att(out_id,var_id,att_nm_tmp,nco_mss_val_sng_get());  
+        continue;
+      }
+
+      /* fill mode and att doesn't exist   */ 
+      if(prs_arg.NCAP4_FILL && rcd_inq_att!=NC_NOERR){
         (void)nco_put_att(out_id,var_id,att_nm_tmp,var_ref->type,1,var_ref->mss_val.vp);
         (void)nco_rename_att(out_id,var_id,att_nm_tmp,nco_mss_val_sng_get());
-      }else{
-        (void)nco_put_att(out_id,var_id,nco_mss_val_sng_get(),var_ref->type,1,var_ref->mss_val.vp);
+        continue;
       }
+
+      /* netcdf3 file just put att */
+      (void)nco_put_att(out_id,var_id,nco_mss_val_sng_get(),var_ref->type,1,var_ref->mss_val.vp);      
       continue;
     }
+   
+
+     /* Write misssing value contained inside variable */
+     /* If missing value type is same as disk type  */  
+     /*
+    if(var_vtr[idx]->xpr_typ == ncap_var ){
+      if(!var_vtr[idx]->var->has_mss_val) continue;  
+      att_item.att_nm=strdup(nco_mss_val_sng_get());
+      att_item.var_nm=strdup(var_vtr[idx]->getVar().c_str());
+      att_item.sz=1;
+      att_item.type=var_vtr[idx]->var->type;
+      att_item.val=var_vtr[idx]->var->mss_val;
+      att_item.mode=aed_overwrite;
+      //att_item.mode=aed_create;
+    }
+     */
+    if(var_vtr[idx]->xpr_typ == ncap_att){
+      /* Skip missing values (for now) */
+      if(var_vtr[idx]->getAtt() == nco_mss_val_sng_get()) continue;     
     
-    /* Skip missing values for now !!! */
-    if(var_vtr[idx]->getAtt() == nco_mss_val_sng_get()) continue;     
-    
-    att_item.att_nm=strdup(var_vtr[idx]->getAtt().c_str());
-    att_item.var_nm=strdup(var_vtr[idx]->getVar().c_str());
-    att_item.sz=var_vtr[idx]->var->sz;
-    att_item.type=var_vtr[idx]->var->type;
-    att_item.val=var_vtr[idx]->var->val;
-    att_item.mode=aed_overwrite;
-    
+      att_item.att_nm=strdup(var_vtr[idx]->getAtt().c_str());
+      att_item.var_nm=strdup(var_vtr[idx]->getVar().c_str());
+      att_item.sz=var_vtr[idx]->var->sz;
+      att_item.type=var_vtr[idx]->var->type;
+      att_item.val=var_vtr[idx]->var->val;
+      att_item.mode=aed_overwrite;
+    } 
+   
+
     if(!strcmp(att_item.var_nm,"global")) 
       var_id=NC_GLOBAL;
-    else {
+    else{
       rcd=nco_inq_varid_flg(out_id,att_item.var_nm,&var_id);
       if(rcd != NC_NOERR)  goto cln_up;
-    }
-    // Check size;
+    } /* end else */
+    /* Check size */
     if(att_item.sz > NC_MAX_ATTRS ){ 
       (void)fprintf(stdout,"%s: Attribute %s size %ld excceeds maximium %d\n",prg_nm_get(),att_item.att_nm,att_item.sz, NC_MAX_ATTRS );
       goto cln_up;
-    }
+    } /* end if */
     /* NB: These attributes should probably be written prior to last data mode */
     (void)nco_aed_prc(out_id,var_id,att_item);
     
-  cln_up: att_item.var_nm=(char*)nco_free(att_item.var_nm);
-    att_item.att_nm=(char*)nco_free(att_item.att_nm);
-  }/* end for */
+ cln_up: ;
+   att_item.var_nm=(char*)nco_free(att_item.var_nm);
+   att_item.att_nm=(char*)nco_free(att_item.att_nm);
+  } /* end for */
   
   /* Set chunksize parameters */
-  if(fl_out_fmt == NC_FORMAT_NETCDF4 || fl_out_fmt == NC_FORMAT_NETCDF4_CLASSIC) (void)nco_cnk_sz_set(out_id,(lmt_all_sct **)NULL_CEWI,(int)0,&cnk_map,&cnk_plc,cnk_sz_scl,cnk,cnk_nbr);
+  if(fl_out_fmt == NC_FORMAT_NETCDF4 || fl_out_fmt == NC_FORMAT_NETCDF4_CLASSIC) (void)nco_cnk_sz_set(out_id,(lmt_msa_sct **)NULL_CEWI,(int)0,&cnk_map,&cnk_plc,cnk_sz_scl,cnk,cnk_nbr);
 
   /* Turn off default filling behavior to enhance efficiency */
   nco_set_fill(out_id,NC_NOFILL,&fll_md_old);
@@ -853,10 +972,10 @@ main(int argc,char **argv)
   } /* end loop over threads */
   
   /* Remove local copy of file */
-  if(FILE_RETRIEVED_FROM_REMOTE_LOCATION && REMOVE_REMOTE_FILES_AFTER_PROCESSING) (void)nco_fl_rm(fl_in);
+  if(FL_RTR_RMT_LCN && RM_RMT_FL_PST_PRC) (void)nco_fl_rm(fl_in);
   
   /* Close output file and move it from temporary to permanent location */
-  if(OUTPUT_TO_NEW_NETCDF_FILE) (void)nco_fl_out_cls(fl_out,fl_out_tmp,out_id); else nco_close(out_id);
+  if(FL_OUT_NEW) (void)nco_fl_out_cls(fl_out,fl_out_tmp,out_id); else nco_close(out_id);
   
   /* Clean memory unless dirty memory allowed */
   if(flg_cln){
@@ -864,7 +983,7 @@ main(int argc,char **argv)
     if(fl_spt_usr) fl_spt_usr=(char *)nco_free(fl_spt_usr);
     
     /* Free extraction lists */ 
-    xtr_lst=nco_nm_id_lst_free(xtr_lst,nbr_xtr);
+    xtr_lst=nco_nm_id_lst_free(xtr_lst,xtr_nbr);
     xtr_lst_a=nco_nm_id_lst_free(xtr_lst_a,nbr_lst_a);
     
     /* Free command line algebraic arguments, if any */
@@ -894,18 +1013,18 @@ main(int argc,char **argv)
     /* Free dimension vectors */
     if(dmn_in_vtr.size() > 0) { 
       for(idx=0;idx<dmn_in_vtr.size();idx++)
-	(void)nco_dmn_free(dmn_in_vtr[idx]);
+        (void)nco_dmn_free(dmn_in_vtr[idx]);
     }
     if(dmn_out_vtr.size() > 0) { 
       for(idx=0;idx< dmn_out_vtr.size();idx++)
-	(void)nco_dmn_free(dmn_out_vtr[idx]);
+        (void)nco_dmn_free(dmn_out_vtr[idx]);
     }
     /* Free var_vtr */
     if(var_vtr.size() > 0) { 
       for(idx=0; idx < var_vtr.size(); idx++)
-	delete var_vtr[idx];
+        delete var_vtr[idx];
     }  
-    
+
     /* Clear vectors */
     /*
     fmc_vtr.clear();
@@ -921,8 +1040,8 @@ main(int argc,char **argv)
 
     */
     /* Free variable lists */
-    if(nbr_xtr > 0) var=nco_var_lst_free(var,nbr_xtr);
-    if(nbr_xtr > 0) var_out=nco_var_lst_free(var_out,nbr_xtr);
+    if(xtr_nbr > 0) var=nco_var_lst_free(var,xtr_nbr);
+    if(xtr_nbr > 0) var_out=nco_var_lst_free(var_out,xtr_nbr);
     var_prc=(var_sct **)nco_free(var_prc);
     var_prc_out=(var_sct **)nco_free(var_prc_out);
     var_fix=(var_sct **)nco_free(var_fix);
@@ -984,5 +1103,35 @@ ram_vars_add
   
   var1=ncap_sclr_var_mk(std::string("__UINT64"),nco_int(NC_UINT64));
   prs_arg->ncap_var_write(var1,true);
+
 #endif // !ENABLE_NETCDF4
+  
+  #ifdef INFINITY
+  var1=ncap_sclr_var_mk(std::string("inff"),INFINITY); //float
+  prs_arg->ncap_var_write(var1,true);
+  #endif
+
+  #ifdef NAN
+  var1=ncap_sclr_var_mk(std::string("nanf"),NAN);    //float
+  prs_arg->ncap_var_write(var1,true);
+  #endif
+
+  #ifdef HUGE_VAL
+  var1=ncap_sclr_var_mk(std::string("inf"),HUGE_VAL);    //double
+  prs_arg->ncap_var_write(var1,true);
+  #endif
+
+  char buff[20];
+  double dnan;
+#ifndef _MSC_VER
+  if( dnan=nan(buff) ){
+    var1=ncap_sclr_var_mk(std::string("nan"),dnan);    //double
+    prs_arg->ncap_var_write(var1,true);
+  }
+#endif
+
+
+
+
+
 } // end ram_vars_add()
