@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco++/prs_cls.cc,v 1.25 2012/02/09 16:03:37 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco++/prs_cls.cc,v 1.29 2013/06/23 19:32:26 zender Exp $ */
 
 /* Purpose: netCDF arithmetic processor */
 /* prs_cls -- symbol table - class methods */
 
-/* Copyright (C) 1995--2012 Charlie Zender
+/* Copyright (C) 1995--2013 Charlie Zender
    License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
@@ -27,9 +27,7 @@
 /* Begin prs_cls methods */
 
 var_sct * 
-prs_cls::ncap_var_init(
-		       const std::string &snm, 
-		       bool bfll){
+prs_cls::ncap_var_init(const std::string &snm,bool bfll){
   
   /* Purpose: Initialize variable structure, retrieve variable values from disk
      Parser calls ncap_var_init() when it encounters a new RHS variable */
@@ -66,7 +64,7 @@ prs_cls::ncap_var_init(
     bfll=false;
   }
   
-  if(dbg_lvl_get() > 2 && !ntl_scn) {
+  if(dbg_lvl_get() >= nco_dbg_fl && !ntl_scn) {
     std::ostringstream os;
     os<< "Parser VAR action called ncap_var_init() to retrieve " <<var_nm <<" from disk";
     dbg_prn(fnc_nm,os.str());  
@@ -84,35 +82,34 @@ prs_cls::ncap_var_init(
   if(!ntl_scn){
     Nvar=var_vtr.find(var_nm);
     
-    // var is defined in O and populated
-    if(Nvar && Nvar->flg_stt==2 && !Nvar->flg_mem){
+    // Variable is defined in O and populated
+    if(Nvar && Nvar->flg_stt == 2 && !Nvar->flg_mem){
       var=Nvar->cpyVarNoData();
       
 #ifdef _OPENMP
-      fl_id= ( omp_in_parallel() ?out_id_readonly : out_id );
+      fl_id=(omp_in_parallel() ? out_id_readonly : out_id);
 #else    
       fl_id=out_id;  
 #endif
       var->tally=(long *)NULL;
+
       /* Retrieve variable values from disk into memory */
-      if(bfll)
-        (void)nco_var_get(fl_id,var); 
+      if(bfll) (void)nco_var_get(fl_id,var); 
       
       return var;
-    }
+    } /* !ntl_scn */
     
-    // var is defined in O and populated && a RAM variable
+    // Variable is defined in O and populated and is a RAM variable
     if(Nvar && Nvar->flg_stt==2 && Nvar->flg_mem){
       var=Nvar->cpyVar();
       return var;
     }
     
-    // var is defined in O but NOT populated
+    // Variable is defined in O but NOT populated
     // Set flag so read is tried only from input
     // Maybe not the best solution ?
     // what else ??? 
-    if(Nvar && Nvar->flg_stt==1 )
-      bskp_npt=true;
+    if(Nvar && Nvar->flg_stt == 1) bskp_npt=true;
     
   } // end !ntl_scn
   
@@ -121,7 +118,7 @@ prs_cls::ncap_var_init(
   if(rcd == NC_NOERR && !bskp_npt){
     
 #ifdef _OPENMP
-    fl_id= ( omp_in_parallel() ?out_id_readonly : out_id );
+    fl_id= (omp_in_parallel() ? out_id_readonly : out_id);
 #else    
     fl_id=out_id;  
 #endif
@@ -129,8 +126,7 @@ prs_cls::ncap_var_init(
     var=nco_var_fll(fl_id,var_id,var_nm,&dmn_out_vtr[0],dmn_out_vtr.size());
     var->tally=(long *)NULL;
     /* Retrieve variable values from disk into memory */
-    if(bfll)
-      (void)nco_var_get(fl_id,var); 
+    if(bfll) (void)nco_var_get(fl_id,var); 
     
     return var;
   }
@@ -146,14 +142,12 @@ prs_cls::ncap_var_init(
     return NULL_CEWI;
   } /* end if */
   
-  /* Find dimensions used in var
+  /* Find dimensions used in variable
      Learn which are not already in output list dmn_out and output file
      Add these to output list and output file */
-  
   fl_id=in_id;
-  
   (void)nco_inq_varndims(fl_id,var_id,&dmn_var_nbr);
-  if(dmn_var_nbr>0){
+  if(dmn_var_nbr > 0){
     int *dim_id=NULL_CEWI;
     dmn_sct *dmn_fd; 
     dmn_sct *dmn_nw;
@@ -175,22 +169,17 @@ prs_cls::ncap_var_init(
         os<<"Unable to find dimension " <<dmn_nm << " in " << fl_in <<" or " << fl_out;
         err_prn(fnc_nm,os.str());
       }
-      
       dmn_tp_out.push_back(dmn_fd);
-      
     } // end idx
     
     // no longer needed
     (void)nco_free(dim_id);
     
     // define new dims in output if necessary  
-    if(dmn_tp_out.size() >0){
-      
+    if(dmn_tp_out.size() > 0){
 #ifdef _OPENMP
-      if( omp_in_parallel())
-	err_prn(fnc_nm,"Attempt to go into netcdf define mode while in OPENMP parallel mode");
+      if(omp_in_parallel()) err_prn(fnc_nm,"Attempt to go into netCDF define mode while in OpenMP parallel mode");
 #endif
-      
       (void)nco_redef(out_id);
       for(idx=0; idx< (int)dmn_tp_out.size();idx++){
         dmn_nw=nco_dmn_dpl(dmn_tp_out[idx]);
@@ -198,18 +187,15 @@ prs_cls::ncap_var_init(
 	(void)nco_dmn_dfn(fl_out,out_id,&dmn_nw,1);          
 	(void)dmn_out_vtr.push_back(dmn_nw);
 	
-	if(dbg_lvl_get() > 2) {
+	if(dbg_lvl_get() >= nco_dbg_fl){
           std::ostringstream os;
-          os << "Found new dimension " << dmn_nw->nm << " in input variable " << var_nm <<" in file " <<fl_in;
+          os << "Found new dimension " << dmn_nw->nm << " in input variable " << var_nm <<" in file " << fl_in;
           os << ". Defining dimension " << dmn_nw->nm << " in output file " << fl_out;
           dbg_prn(fnc_nm,os.str());
 	}
-      }// end idx
-      
+      } // end idx
       (void)nco_enddef(out_id);
-      
     } // end if 
-    
   } // end if
   
   var=nco_var_fll(fl_id,var_id,var_nm,&dmn_out_vtr[0],dmn_out_vtr.size());
@@ -247,13 +233,11 @@ prs_cls::ncap_var_init_chk(
   
   // Check output file for ID 
   rcd=nco_inq_varid_flg(out_id,var_nm.c_str(),&var_id);
-  if(rcd == NC_NOERR)
-    return 1;
+  if(rcd == NC_NOERR) return 1;
   
   // Check input file for ID 
   rcd=nco_inq_varid_flg(in_id,var_nm.c_str(),&var_id);
-  if(rcd == NC_NOERR)
-    return 1;
+  if(rcd == NC_NOERR) return 1;
   
   // Variable not in Input or Output or int_vtr
   return 0;
@@ -335,8 +319,7 @@ prs_cls::ncap_var_write_omp(
 #ifdef NCO_RUSAGE_DBG
   long maxrss; /* [B] Maximum resident set size */
 #endif /* !NCO_RUSAGE_DBG */
-  
-  
+    
   // INITIAL SCAN
   if(ntl_scn){
     Nvar=var_vtr.find(var->nm);
@@ -465,20 +448,14 @@ prs_cls::ncap_var_write_omp(
     var_inf=nco_var_free(var_inf);
     
     var_out_id=var->id;
-    
   } 
-  
-  
   rcd=nco_inq_varid_flg(out_id,var->nm,&var_out_id);
   
-  
   // Only go into define mode if necessary
   if(!bdef || var->pck_ram ){  
     
 #ifdef _OPENMP
-    if( omp_in_parallel())
-      err_prn(fnc_nm, "Attempt to go into netcdf define mode while in OPENMP parallel mode");
-    
+    if(omp_in_parallel()) err_prn(fnc_nm, "Attempt to go into netCDF define mode while in OpenMP parallel mode");
 #endif
     (void)nco_redef(out_id);
     
@@ -525,12 +502,12 @@ prs_cls::ncap_var_write_omp(
 #ifdef NCO_RUSAGE_DBG
   /* Compile: cd ~/nco/bld;make 'USR_TKN=-DNCO_RUSAGE_DBG';cd - */
   /* Print rusage memory usage statistics */
-  if(dbg_lvl_get() >= 0) {
+  if(dbg_lvl_get() >= nco_dbg_fl) {
     std::ostringstream os;
     os<<" Writing variable "<<var_nm; <<" to disk.";
     dbg_prn(fnc_nm,os.str());
   }
-  maxrss=nco_mmr_rusage_prn((int)0);
+  maxrss=nco_mmr_usg_prn((int)0);
 #endif /* !NCO_RUSAGE_DBG */
   
   // save variable to output vector if new
@@ -567,7 +544,7 @@ void prs_cls::ncap_def_ntl_scn(void)
   
   const std::string fnc_nm("prs_cls::ncap_def_ntl_scn"); 
   
-  if(dbg_lvl_get() > 0) dbg_prn(fnc_nm, "Entered function");
+  if(dbg_lvl_get() >= nco_dbg_scl) dbg_prn(fnc_nm, "Entered function");
   
   sz=int_vtr.size();
   
@@ -577,7 +554,7 @@ void prs_cls::ncap_def_ntl_scn(void)
     var1=Nvar->var;
     if(!Nvar->flg_udf && Nvar->xpr_typ==ncap_var){
       
-      if(dbg_lvl_get() > 0) dbg_prn(fnc_nm, Nvar->getFll()+ (!Nvar->flg_mem ? " defined in output": " RAM variable"));
+      if(dbg_lvl_get() >= nco_dbg_scl) dbg_prn(fnc_nm, Nvar->getFll()+ (!Nvar->flg_mem ? " defined in output": " RAM variable"));
       
       // Define variable
       if(!Nvar->flg_mem){
