@@ -1,8 +1,8 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_att_utl.h,v 1.35 2012/01/01 20:51:53 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_att_utl.h,v 1.47 2013/07/18 20:11:04 zender Exp $ */
 
 /* Purpose: Attribute utilities */
 
-/* Copyright (C) 1995--2012 Charlie Zender
+/* Copyright (C) 1995--2013 Charlie Zender
    License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
@@ -18,7 +18,10 @@
 
 /* Standard header files */
 #include <stdio.h> /* stderr, FILE, NULL, printf */
-#include <string.h> /* strcmp. . . */
+#include <string.h> /* strcmp() */
+#ifdef HAVE_STRINGS_H
+# include <strings.h> /* strcasecmp() */
+#endif /* !HAVE_STRINGS_H */
 #include <time.h> /* machine time */
 
 /* 3rd party vendors */
@@ -32,6 +35,9 @@
 #include "nco_lst_utl.h" /* List utilities */
 #include "nco_mmr.h" /* Memory management */
 #include "nco_mss_val.h" /* Missing value utilities */
+#ifdef _MSC_VER
+# include "nco_rth_flt.h" /* Float-precision arithmetic, MSVC macros */
+#endif /* !_MSC_VER */
 #include "nco_scm.h" /* Software configuration management */
 #include "nco_sng_utl.h" /* String utilities */
 
@@ -77,7 +83,30 @@ nco_prs_aed_lst /* [fnc] Parse user-specified attribute edits into structure lis
 int /* [flg] Variable and attribute names are conjoined */
 nco_prs_att /* [fnc] Parse conjoined variable and attribute names */
 (rnm_sct * const rnm_att, /* I/O [sct] Structure [Variable:]Attribute name on input, Attribute name on output */
- char * const var_nm); /* O [sng] Variable name, if any */
+ char * const var_nm, /* O [sng] Variable name, if any */
+ nco_bool * const IS_GLB_GRP_ATT); /* O [flg] Attribute is Global or Group attribute */
+
+gpe_sct * /* O [sng] GPE structure */
+nco_gpe_prs_arg /* [fnc] Parse Group Path Editing (GPE) argument */
+(const char * const gpe_arg); /* I [sng] User-specified GPE specification */
+
+char * /* O [sng] Result of applying GPE to input path */
+nco_gpe_evl /* [fnc] Apply Group Path Editing (GPE) to argument */
+(const gpe_sct * const gpe, /* I [sng] GPE structure */
+ const char * const grp_nm_fll_in); /* I [sng] Full group name */
+
+char * /* O [sng] Stub of GPE applied to input path */
+nco_gpe_evl_stb /* [fnc] Find stub of Group Path Editing (GPE) applied to argument */
+(const gpe_sct * const gpe, /* I [sng] GPE structure */
+ const char * const grp_nm_fll_in); /* I [sng] Full group name */
+
+gpe_sct * /* O [sct] Structure with dynamic memory free()'d */
+nco_gpe_free /* [fnc] Free dynamic memory of GPE structure */
+(gpe_sct * const gpe); /* O [sct] GPE structure */
+
+const char * /* O [sng] String describing GPE */
+nco_gpe_sng /* [fnc] Convert GPE enum to string */
+(const gpe_enm gpe_md); /* I [enm] GPE mode */
 
 rnm_sct * /* O [sng] Structured list of old, new names */
 nco_prs_rnm_lst /* [fnc] Set old_nm, new_nm elements of rename structure */
