@@ -1,8 +1,8 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_cnf_typ.h,v 1.34 2012/01/01 20:51:53 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_cnf_typ.h,v 1.40 2013/07/16 18:39:43 zender Exp $ */
 
 /* Purpose: Conform variable types */
 
-/* Copyright (C) 1995--2012 Charlie Zender
+/* Copyright (C) 1995--2013 Charlie Zender
    License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
@@ -15,7 +15,7 @@
 /* Standard header files */
 #include <math.h> /* sin cos cos sin 3.14159 */
 #include <stdio.h> /* stderr, FILE, NULL, printf */
-#include <string.h> /* strcmp. . . */
+#include <string.h> /* strcmp() */
 
 /* 3rd party vendors */
 #include <netcdf.h> /* netCDF definitions and C library */
@@ -24,7 +24,9 @@
 /* Personal headers */
 #include "nco.h" /* netCDF Operator (NCO) definitions */
 #include "nco_mmr.h" /* Memory management */
-
+#ifdef _MSC_VER
+# include "nco_rth_flt.h" /* Float-precision arithmetic, MSVC macros */
+#endif 
 
 #ifdef __cplusplus
 extern "C" {
@@ -54,6 +56,11 @@ nco_var_cnf_typ /* [fnc] Return copy of input variable typecast to desired type
 (const nc_type var_out_typ, /* I [enm] Type to convert variable structure to */
  var_sct * const var_in); /* I/O [enm] Pointer to variable structure (may be destroyed) */
 
+var_sct * /* O [sct] Pointer to variable structure of type var_out_typ */
+nco_var_cnf_typ_tst /* [fnc] Return copy of input variable typecast to desired type */
+(const nc_type var_out_typ, /* I [enm] Type to convert variable structure to */
+ var_sct * const var_in); /* I/O [enm] Pointer to variable structure (may be destroyed) */
+
 var_sct * /* O [sct] Variable with mss_val converted to typ_upk */
 nco_cnv_mss_val_typ  /* [fnc] Convert missing_value, if any, to mss_val_out_typ */
 (var_sct *var, /* I [sct] Variable with missing_value to convert */
