@@ -1,8 +1,8 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_dmn_utl.h,v 1.35 2012/01/01 20:51:53 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_dmn_utl.h,v 1.47 2013/06/17 05:24:33 pvicente Exp $ */
 
 /* Purpose: Dimension utilities */
 
-/* Copyright (C) 1995--2012 Charlie Zender
+/* Copyright (C) 1995--2013 Charlie Zender
    License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
@@ -18,7 +18,7 @@
 
 /* Standard header files */
 #include <stdio.h> /* stderr, FILE, NULL, printf */
-#include <string.h> /* strcmp. . . */
+#include <string.h> /* strcmp() */
 
 /* 3rd party vendors */
 #include <netcdf.h> /* netCDF definitions and C library */
@@ -28,6 +28,7 @@
 #include "nco.h" /* netCDF Operator (NCO) definitions */
 #include "nco_mmr.h" /* Memory management */
 #include "nco_sng_utl.h" /* String utilities */
+#include "nco_grp_utl.h"  /* Group utilities */
 
 #ifdef __cplusplus
 extern "C" {
@@ -75,15 +76,6 @@ nco_dmn_lmt_mrg /* [fnc] Merge limit structure information into dimension struct
  CST_X_PTR_CST_PTR_CST_Y(lmt_sct,lmt), /* I [sct] Dimension limit information */
  const int lmt_nbr); /* I [nbr] Number of dimensions with user-specified limits */
 
-
-void
-nco_dmn_lmt_all_mrg /* [fnc] Merge limit structure information into dimension structures */
-(dmn_sct ** const dmn, /* I [sct] Dimension structures to modify */
- const int nbr_dmn, /* I [nbr] Number of dimension structures in structure list */
- CST_X_PTR_CST_PTR_CST_Y(lmt_all_sct,lmt_all_lst), /* I [sct] Dimension limit information */
- const int lmt_nbr); /* I [nbr] Number of dimensions with user-specified limits */
-
-
 dmn_sct ** /* O [sct] Pointer to free'd structure list */
 nco_dmn_lst_free /* [fnc] Free memory associated with dimension structure list */
 (dmn_sct **dmn_lst, /* I/O [sct] Dimension structure list to free */
@@ -95,6 +87,18 @@ nco_dmn_lst_mk /* [fnc] Attach dimension IDs to dimension list */
  CST_X_PTR_CST_PTR_CST_Y(char,dmn_lst_in), /* I [sng] User-specified list of dimension names */
  const int nbr_dmn); /* I [nbr] Total number of dimensions in list */
 
+void
+nco_dmn_xrf  /* [fnc] Crossreference xrf elements of dimension structures */
+(dmn_sct * const dmn_1, /* I/O [sct] Dimension structure */
+ dmn_sct * const dmn_2); /* I/O [sct] Dimension structure */
+
+int /* O [flg] Dimension exists in scope of group (if rcd != NC_NOERR) */
+nco_inq_dmn_grp_id /* [fnc] Return location and ID of named dimension in specified group */
+(const int nc_id, /* I [id] netCDF group ID */
+ const char * const dmn_nm, /* I [sng] Dimension name */
+ int * const dmn_id, /* O [id] Dimension ID in specified group */
+ int * const grp_id_dmn); /* O [id] Group ID where dimension visible to specified group is defined */
+
 nm_id_sct * /* O [sct] List of dimensions associated with input variable list */
 nco_dmn_lst_ass_var /* [fnc] Create list of all dimensions associated with input variable list */
 (const int nc_id, /* I [id] netCDF input-file ID */
@@ -102,21 +106,21 @@ nco_dmn_lst_ass_var /* [fnc] Create list of all dimensions associated with input
  const int nbr_var, /* I [nbr] Number of variables in list */
  int * const nbr_dmn); /* O [nbr] Number of dimensions associated with input variable list */
 
-void
-nco_dmn_xrf  /* [fnc] Crossreference xrf elements of dimension structures */
-(dmn_sct * const dmn_1, /* I/O [sct] Dimension structure */
- dmn_sct * const dmn_2); /* I/O [sct] Dimension structure */
-
-
 void 
-nco_dmn_sct_cmp   /* [fnc] Check that dims in list 2 are a subset of list 1 and that they are the same size */
-(dmn_sct ** const dim_1, /* I [sct] Dimension list 1 */
- const int nbr_dmn_1,  /* I [nbr] Number of dimension structures in structure list */
- dmn_sct **const dim_2,  /* I [sct] Dimension list 1 */
- const int nbr_dmn_2,  /* I [nbr] Number of dimension structures in structure list */
+nco_dmn_sct_cmp /* [fnc] Check that dims in list 2 are a subset of list 1 and that they are the same size */
+(dmn_sct ** const dim_1n, /* I [sct] Dimension list 1 */
+ const int nbr_dmn_1, /* I [nbr] Number of dimension structures in structure list */
+ dmn_sct **const dim_2, /* I [sct] Dimension list 1 */
+ const int nbr_dmn_2, /* I [nbr] Number of dimension structures in structure list */
  const char *const fl_sng_1, /* I [sng] Name of first file */
- const char *fl_sng_2);     /* I [sng] Name of second file */
+ const char *fl_sng_2); /* I [sng] Name of second file */
 
+void
+nco_dmn_lmt_all_mrg /* [fnc] Merge limit structure information into dimension structures */
+(dmn_sct ** const dmn, /* I [sct] Dimension structures to modify */
+ const int nbr_dmn, /* I [nbr] Number of dimension structures in structure list */
+ CST_X_PTR_CST_PTR_CST_Y(lmt_msa_sct,lmt_all_lst), /* I [sct] Dimension limit information */
+ const int lmt_nbr); /* I [nbr] Number of dimensions with user-specified limits */
 
 
 #ifdef __cplusplus
