@@ -1,8 +1,8 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_fl_utl.h,v 1.46 2012/01/01 20:51:53 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_fl_utl.h,v 1.72 2013/01/19 04:29:16 zender Exp $ */
 
 /* Purpose: File manipulation */
 
-/* Copyright (C) 1995--2012 Charlie Zender
+/* Copyright (C) 1995--2013 Charlie Zender
    License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
@@ -17,19 +17,29 @@
 #endif /* !HAVE_CONFIG_H */
 
 /* Standard header files */
+#include <errno.h> /* system/library error diagnostics, errno */
 #include <math.h> /* sin cos cos sin 3.14159 */
 #include <stdio.h> /* stderr, FILE, NULL, printf */
 #include <stdlib.h> /* strtod, strtol, malloc, getopt, exit */
-#include <string.h> /* strcmp. . . */
+#include <string.h> /* strcmp() */
+#ifdef HAVE_STRINGS_H
+# include <strings.h> /* strcasecmp() */
+#endif /* !HAVE_STRINGS_H */
 #include <sys/stat.h> /* stat() */
-#include <unistd.h> /* POSIX stuff */
+#ifndef _MSC_VER
+# include <unistd.h> /* POSIX stuff */
+# include <netinet/in.h> /* needed for _res */
+# include <pwd.h> /* password structures for getpwuid() */
+#endif /* !_MSC_VER */
+#ifdef _MSC_VER
+# include <process.h> /* MSVC getpid() */
+typedef int pid_t;
+#endif /* !_MSC_VER */
 
 #include <sys/types.h> /* needed for _res */
-#include <netinet/in.h> /* needed for _res */
-#include <pwd.h> /* password structures for getpwuid() */
 #ifndef WIN32
-#include <arpa/nameser.h> /* needed for _res */
-#include <resolv.h> /* Internet structures for _res */
+# include <arpa/nameser.h> /* needed for _res */
+# include <resolv.h> /* Internet structures for _res */
 #endif /* !WIN32 */
 
 /* 3rd party vendors */
@@ -42,13 +52,23 @@
 #include "nco_mmr.h" /* Memory management */
 #include "nco_sng_utl.h" /* String utilities */
 
+/* 20120301: Replace multiple instances of hard-coded retry limit with CPP token NCO_MAX_NBR_USR_INPUT_RETRY */
+#ifndef NCO_MAX_NBR_USR_INPUT_RETRY
+# define NCO_MAX_NBR_USR_INPUT_RETRY 10
+#endif /* NCO_MAX_NBR_USR_INPUT_RETRY */
+
+/* 20120301: Replace multiple instances of hard-coded reply length limit with CPP token NCO_USR_RPL_MAX_LNG */
+#ifndef NCO_USR_RPL_MAX_LNG
+# define NCO_USR_RPL_MAX_LNG 10
+#endif /* NCO_USR_RPL_MAX_LNG */
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
 int /* O [enm] Mode flag for nco_create() call */
 nco_create_mode_mrg /* [fnc] Merge clobber mode with user-specified file format */
-(const int clobber_mode, /* I [enm] Clobber mode (NC_CLOBBER or NC_NOCLOBBER) */
+(const int md_clobber, /* I [enm] Clobber mode (NC_CLOBBER or NC_NOCLOBBER) */
  const int fl_out_fmt); /* I [enm] Output file format */
 
 int /* [rcd] Return code */
@@ -59,11 +79,19 @@ nco_create_mode_prs /* [fnc] Parse user-specified file format */
 void
 nco_fl_cmp_err_chk(void); /* [fnc] Perform error checking on file */
 
+void
+nco_fl_chmod /* [fnc] Ensure file is user/owner-writable */
+(const char * const fl_nm); /* I [sng] Name of file */
+
 void
 nco_fl_cp /* [fnc] Copy first file to second */
 (const char * const fl_src, /* I [sng] Name of source file to copy */
  const char * const fl_dst); /* I [sng] Name of destination file */
 
+void
+nco_fl_overwrite_prm /* [fnc] Obtain user consent to overwrite output file */
+(const char * const fl_nm); /* I [sng] Name of file */
+
 void
 nco_fl_fmt_vet /* [fnc] Verify output file format supports requested actions */
 (const int fl_fmt, /* I [enm] Output file format */
@@ -87,7 +115,7 @@ char * /* O [sng] Filename of locally available file */
 nco_fl_mk_lcl /* [fnc] Retrieve input file and return local filename */
 (char *fl_nm, /* I/O [sng] Current filename, if any (destroyed) */
  const char * const fl_pth_lcl, /* I [sng] Local storage area for files retrieved from remote locations */
- nco_bool * const FILE_RETRIEVED_FROM_REMOTE_LOCATION); /* O [flg] File was retrieved from remote location */
+ nco_bool * const FL_RTR_RMT_LCN); /* O [flg] File was retrieved from remote location */
 
 void
 nco_fl_mv /* [fnc] Move first file to second */
@@ -110,6 +138,10 @@ nco_fl_out_open /* [fnc] Open output file subject to availability and user input
  const nco_bool FORCE_APPEND, /* I [flg] Append to existing file, if any */
  const nco_bool FORCE_OVERWRITE, /* I [flg] Overwrite existing file, if any */
  const int fl_out_fmt, /* I [enm] Output file format */
+ const size_t * const bfr_sz_hnt, /* I [B] Buffer size hint */
+ const int RAM_CREATE, /* I [flg] Create file in RAM */
+ const int RAM_OPEN, /* I [flg] Open (netCDF3) file(s) in RAM */
+ const int WRT_TMP_FL, /* I [flg] Write output to temporary file */
  int * const out_id); /* O [id] File ID */
 
 void
@@ -122,6 +154,12 @@ void
 nco_fl_rm /* [fnc] Remove file */
 (char *fl_nm); /* I [sng] File to be removed */
 
+int /* [rcd] Return code */
+nco_fl_open /* [fnc] Open file using appropriate buffer size hints and verbosity */
+(const char * const fl_nm, /* I [sng] Name of file to open */
+ const int md_open, /* I [enm] Mode flag for nc_open() call */
+ const size_t * const bfr_sz_hnt, /* I [B] Buffer size hint */
+ int * const nc_id); /* O [id] File ID */
 
 #ifdef __cplusplus
 } /* end extern "C" */
