@@ -1,21 +1,48 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_netcdf.h,v 1.80 2012/01/01 20:51:53 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_netcdf.h,v 1.102 2013/08/01 05:02:07 zender Exp $ */
 
 /* Purpose: NCO wrappers for netCDF C library */
 
-/* Copyright (C) 1995--2012 Charlie Zender
+/* Copyright (C) 1995--2013 Charlie Zender
    License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
 #include "nco_netcdf.h" *//* NCO wrappers for netCDF C library */
 
+/* nco_netcdf.h is (nearly) independent of NCO and does not depend on nco.h
+   nco_netcdf.h is an abstraction layer for netcdf.h, plus a few convenience routines
+   A similar abstraction layer must exist for each NCO storage backend, e.g., nco_hdf.h
+
+   1. Utility routines, e.g., nco_typ_lng(), c_typ_nm() (routines with no netCDF library counterpart)
+   2. File-routine wrappers, e.g., nco_open()
+   3. Dimension-routine wrappers, e.g., nco_dimid()
+   4. Variable-routine wrappers, e.g., nco_get_var()
+   5. Attribute-routine wrappers, e.g., nco_put_att()
+
+   Name Convention: Where appropriate, routine name is identical to netCDF C-library name,
+   except nc_ is replaced by nco_
+
+   Argument Ordering Convention: Routines follow same argument order as netCDF C-library
+   Additional arguments, such as nc_type, are appended to end of argument list
+
+   Return value convention: Functions return a success/failure code
+   Errors in netCDF functions cause NCO wrapper to abort, except *_flg() wrappers
+   *_flg() wrappers allow limited, pre-defined, netCDF errors to be non-fatal
+
+   Circularity convention: It is important that none of the error diagnostics
+   in these functions call other nco_??? routines because if everything
+   starts failing then errors will produce circular diagnostics.
+   To ensure this is the case, it is only safe to print diagnostics on
+   variables which are supposed to be valid on input. */
+
 #ifndef NCO_NETCDF_H /* Contents have not yet been inserted in current source file */
 #define NCO_NETCDF_H
 
 /* Standard header files */
 #include <stdio.h> /* stderr, FILE, NULL, printf */
 #include <stdlib.h> /* strtod, strtol, malloc, getopt, exit */
-#include <string.h> /* strcmp. . . */
+#include <string.h> /* strcmp() */
+#include <assert.h> /* assert() */
 
 /* 3rd party vendors */
 #include <netcdf.h> /* netCDF definitions and C library */
@@ -27,6 +54,58 @@
 /* Personal headers */
 #include "nco_typ.h" /* Type definitions, opaque types */
 
+/* NB: 
+   nco_netcdf.h does #include nco_typ.h which #defines some forward-compatibility tokens
+   nco_netcdf.h does not #include nco.h which #defines some forward-compatibility tokens
+   This barrier helps segregate NCO from wrappers
+   Re-define minimal sub-set of tokens for nco_netcdf.c as necessary
+   This is small exception to the barrier */
+#ifndef NC_FORMAT_CLASSIC
+# define NC_FORMAT_CLASSIC (1)
+#endif
+#ifndef NC_FORMAT_64BIT
+# define NC_FORMAT_64BIT   (2)
+#endif
+#ifndef NC_FORMAT_NETCDF4
+# define NC_FORMAT_NETCDF4 (3)
+#endif
+#ifndef NC_FORMAT_NETCDF4_CLASSIC
+# define NC_FORMAT_NETCDF4_CLASSIC  (4) /* create netcdf-4 files, with NC_CLASSIC_MODEL. */
+#endif
+#ifndef NC_CHUNKED
+# define NC_CHUNKED (0)
+#endif
+#ifndef NC_CONTIGUOUS
+# define NC_CONTIGUOUS (1)
+#endif
+#ifndef NC_NOCHECKSUM
+# define NC_NOCHECKSUM 0
+#endif
+
+/* Some netCDF3 stubs for netCDF4 routines need netCDF4-only return codes
+   These netCDF4 tokens are never defined in netCDF3-only netcdf.h */
+#ifndef HAVE_NETCDF4_H
+# ifndef NC_EBADGRPID
+#  define NC_EBADGRPID (-116)    /**< Bad group ID. */
+# endif
+# ifndef NC_ENOGRP
+#  define NC_ENOGRP        (-125)    /**< No group found. */
+# endif
+#endif /* HAVE_NETCDF4_H */
+
+/* netcdf.h replaced NC_EFILLVALUE by case NC_ELATEFILL after about netCDF ~4.2.1 */
+#ifdef HAVE_NETCDF4_H
+# ifndef NC_EFILLVALUE
+#  define NC_EFILLVALUE NC_ELATEFILL
+# endif /* NC_EFILLVALUE */
+# ifndef NC_ELATEFILL
+#  define NC_ELATEFILL NC_EFILLVALUE
+# endif /* NC_ELATEFILL */
+#else /* !HAVE_NETCDF4_H */
+# define NC_ELATEFILL     (-122)    /**< Attempt to define fill value when data already exists. */
+# define NC_EFILLVALUE    (-122)    /**< Attempt to define fill value when data already exists. */
+#endif /* !HAVE_NETCDF4_H */
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -36,15 +115,23 @@ extern "C" {
 /* Begin Utility Routines */
 const char * /* O [sng] Native C type */
 c_typ_nm /* [fnc] Return string describing native C type */
-(const nc_type type); /* O [enm] netCDF type */
+(const nc_type type); /* I [enm] netCDF type */
+
+const char * /* O [sng] Native CDL type */
+cdl_typ_nm /* [fnc] Return string describing native CDL type */
+(const nc_type type); /* I [enm] netCDF type */
+
+const char * /* O [sng] Native CDL type suffix */
+cdl_typ_sfx /* [fnc] Return suffix string for CDL type */
+(const nc_type type); /* I [enm] netCDF type */
 
 const char * /* O [sng] Native Fortran77 type */
 f77_typ_nm /* [fnc] Return string describing native Fortran77 type */
-(const nc_type type); /* O [enm] netCDF type */
+(const nc_type type); /* I [enm] netCDF type */
 
 const char * /* O [sng] Native Fortran90 type */
 f90_typ_nm /* [fnc] Return string describing native Fortran90 type */
-(const nc_type type); /* O [enm] netCDF type */
+(const nc_type type); /* I [enm] netCDF type */
 
 void 
 nco_dfl_case_nc_type_err /* [fnc] Print error and exit for illegal switch(nc_type) case */
@@ -92,8 +179,10 @@ int ncompi_open(MPI_Comm mpi_cmm,const char * const fl_nm,const int mode,MPI_Inf
 
 /* Begin file-level routines */
 int nco_create(const char * const fl_nm,const int cmode,int * const nc_id);
+int nco__create(const char * const fl_nm,const int cmode,const size_t sz_ntl,size_t * const bfr_sz_hnt,int * const nc_id);
 int nco_open(const char * const fl_nm,const int mode,int * const nc_id);
 int nco_open_flg(const char * const fl_nm,const int mode,int * const nc_id);
+int nco__open(const char * const fl_nm,const int mode,size_t * const bfr_sz_hnt,int * const nc_id);
 int nco_redef(const int nc_id);
 int nco_set_fill(const int nc_id,const int fill_mode,int * const old_mode);
 int nco_enddef(const int nc_id);
@@ -107,9 +196,8 @@ int nco_inq(const int nc_id,int * const dmn_nbr_fl,int * const var_nbr_fl,int *
 int nc_inq_format(int nc_id, int * const fl_fmt);
 #endif /* !NEED_NC_INQ_FORMAT */
 int nco_inq_format(const int nc_id,int * const fl_fmt);
-#ifdef ENABLE_NETCDF4
 int nco_inq_ncid(const int nc_id,const char * const grp_nm,int * const grp_id);
-#endif /* !ENABLE_NETCDF4 */
+int nco_inq_ncid_flg(const int nc_id,const char * const grp_nm,int * const grp_id);
 int nco_inq_ndims(const int nc_id,int * const dmn_nbr_fl);
 int nco_inq_nvars(const int nc_id,int * const var_nbr_fl);
 int nco_inq_natts(const int nc_id,int * const att_glb_nbr);
@@ -118,18 +206,21 @@ int nco_inq_unlimdim_flg(const int nc_id,int * const rec_dmn_id);
 /* End File routines */
 
 /* Begin Group routines (_grp) */
-#ifdef ENABLE_NETCDF4
+int nco_rename_grp(int grp_id,const char * const grp_nm);
 int nco_def_grp(const int nc_id,const char * const grp_nm,int * const grp_id);
+int nco_def_grp_flg(const int nc_id,const char * const grp_nm,int * const grp_id);
 int nco_inq_dimids(const int nc_id,int * const dmn_nbr,int * const dmn_ids,int flg_prn);
 int nco_inq_grpname(const int nc_id,char * const grp_nm);
 int nco_inq_grpname_full(const int nc_id,size_t * grp_nm_lng,char * const grp_nm_fll);
 int nco_inq_grpname_len(const int nc_id,size_t * const grp_nm_lng);
 int nco_inq_grps(const int nc_id,int * const grp_nbr,int * const grp_ids);
-int nco_inq_grp_full_ncid(const int nc_id,char * const grp_nm_fll,int * const grp_id);
-int nco_inq_grp_ncid(const int nc_id,char * const grp_nm,int * const grp_id);
+int nco_inq_grp_full_ncid(const int nc_id,const char * const grp_nm_fll,int * const grp_id);
+int nco_inq_grp_full_ncid_flg(const int nc_id,const char * const grp_nm_fll,int * const grp_id);
+int nco_inq_grp_ncid(const int nc_id,const char * const grp_nm,int * const grp_id);
+int nco_inq_grp_ncid_flg(const int nc_id,const char * const grp_nm,int * const grp_id);
 int nco_inq_grp_parent(const int nc_id,int * const prn_id);
+int nco_inq_grp_parent_flg(const int nc_id,int * const prn_id);
 int nco_inq_varids(const int nc_id,int * const var_nbr,int * const var_ids);
-#endif /* !ENABLE_NETCDF4 */
 /* End Group routines */
 
 /* Begin Dimension routines (_dim) */
@@ -141,6 +232,7 @@ int nco_inq_dim_flg(const int nc_id,const int dmn_id,char *dmn_nm,long *dmn_sz);
 int nco_inq_dimname(const int nc_id,const int dmn_id,char *dmn_nm);
 int nco_inq_dimlen(const int nc_id,const int dmn_id,long *dmn_sz);
 int nco_rename_dim(const int nc_id,const int dmn_id,const char * const dmn_nm);
+int nco_inq_unlimdims(const int nc_id,int *nbr_dmn_ult,int *dmn_ids_ult);
 /* End Dimension routines */
 
 /* Begin Variable routines (_var) */
@@ -189,6 +281,11 @@ int nco_put_att(const int nc_id,const int var_id,const char * const att_nm,const
 int nco_get_att(const int nc_id,const int var_id,const char * const att_nm,void * const vp,const nc_type att_typ);
 /* End Attribute routines */
 
+/* Begin libnetcdf prototypes (i.e., for completely new library routines not yet in netcdf.h */
+#ifndef NC_HAVE_RENAME_GRP
+  int nc_rename_grp(int grp_id,const char * const grp_nm);
+#endif /* NC_HAVE_RENAME_GRP */
+
 /* Begin netCDF4 stubs */
 #ifndef HAVE_NETCDF4_H
   /* Stubs so netCDF4 functions work without protection in netCDF3 environments */
