@@ -1,8 +1,8 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_rth_utl.c,v 1.43 2012/01/01 20:51:53 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_rth_utl.c,v 1.57 2013/09/19 03:35:18 pvicente Exp $ */
 
 /* Purpose: Arithmetic controls and utilities */
 
-/* Copyright (C) 1995--2012 Charlie Zender
+/* Copyright (C) 1995--2013 Charlie Zender
    License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
@@ -33,9 +33,87 @@ nco_rth_prc_rnk /* [fnc] Rank precision of arithmetic type */
   return (nco_rth_prc_rnk_enm)0;
 } /* end nco_rth_prc_rnk() */
 
+void 
+nco_opr_nrm /* [fnc] Normalization of arithmetic operations for ncra/ncea */
+(const int nco_op_typ, /* I [enm] Operation type */
+ const int nbr_var_prc, /* I [nbr] Number of processed variables */
+ X_CST_PTR_CST_PTR_Y(var_sct,var_prc), /* I [sct] Variables in input file */
+ X_CST_PTR_CST_PTR_Y(var_sct,var_prc_out), /* I/O [sct] Variables in output file */
+ const nco_bool flg_nrm, /* I [flg] This record needs normalization */
+ const char * const rec_nm_fll,      /* I [sng] Full name of record being done in loop (trv_tbl->lmt_rec[idx_rec]->nm_fll ) */
+ const trv_tbl_sct * const trv_tbl) /* I [sct] Traversal table */
+{
+  /* Purpose: Normalize appropriate ncra/ncea operation (avg, min, max, ttl, ...) on operands
+     Values of var_prc are not altered but are not const because missing values are cast
+     Values of var_prc_out are altered (i.e., normalized) */
+
+  int idx=int_CEWI;
+  int nbr_var_prc_cpy;
+  int nco_op_typ_cpy;
+
+  nco_op_typ_cpy=nco_op_typ;
+  nbr_var_prc_cpy=nbr_var_prc;
+
+#ifdef _OPENMP
+#pragma omp parallel for default(none) private(idx) shared(nbr_var_prc_cpy,nco_op_typ_cpy,var_prc,var_prc_out)
+#endif /* !_OPENMP */
+  for(idx=0;idx<nbr_var_prc_cpy;idx++){
+
+    /* Skip variable if does not relate to current record */
+    if (rec_nm_fll){
+      nco_bool flg_skp=nco_skp_var(var_prc[idx],rec_nm_fll,trv_tbl);
+      if (flg_skp){
+        continue;
+      }
+    }
+
+    if(var_prc[idx]->is_crd_var){
+      /* Return linear averages of coordinates unless computing extrema
+      Prevent coordinate variables from encountering nco_var_nrm_sdn() */
+      (void)nco_var_nrm(var_prc_out[idx]->type,var_prc_out[idx]->sz,var_prc[idx]->has_mss_val,var_prc[idx]->mss_val,var_prc[idx]->tally,var_prc_out[idx]->val);
+    }else{ /* !var_prc[idx]->is_crd_var */
+      switch(nco_op_typ_cpy){
+      case nco_op_avg: /* Normalize sum by tally to create mean */
+      case nco_op_sqrt: /* Normalize sum by tally to create mean */
+      case nco_op_sqravg: /* Normalize sum by tally to create mean */
+      case nco_op_rms: /* Normalize sum of squares by tally to create mean square */
+      case nco_op_avgsqr: /* Normalize sum of squares by tally to create mean square */
+        (void)nco_var_nrm(var_prc_out[idx]->type,var_prc_out[idx]->sz,var_prc[idx]->has_mss_val,var_prc[idx]->mss_val,var_prc[idx]->tally,var_prc_out[idx]->val);
+
+        break;
+      case nco_op_rmssdn: /* Normalize sum of squares by tally-1 to create mean square for sdn */
+        (void)nco_var_nrm_sdn(var_prc_out[idx]->type,var_prc_out[idx]->sz,var_prc[idx]->has_mss_val,var_prc[idx]->mss_val,var_prc[idx]->tally,var_prc_out[idx]->val);
+        break;
+      case nco_op_min: /* Minimum is already in buffer, do nothing */
+      case nco_op_max: /* Maximum is already in buffer, do nothing */
+        break;
+      case nco_op_ttl: /* Total is already in buffer, stuff missing values into elements with zero tally */
+        (void)nco_var_tll_zro_mss_val(var_prc_out[idx]->type,var_prc_out[idx]->sz,var_prc[idx]->has_mss_val,var_prc[idx]->mss_val,var_prc[idx]->tally,var_prc_out[idx]->val);
+        break;
+      default:
+        break;
+      } /* end switch */
+      /* Some operations require additional processing */
+      switch(nco_op_typ_cpy) {
+      case nco_op_rms: /* Take root of mean of sum of squares to create root mean square */
+      case nco_op_rmssdn: /* Take root of sdn mean of sum of squares to create root mean square for sdn */
+      case nco_op_sqrt: /* Take root of mean to create root mean */
+        (void)nco_var_sqrt(var_prc_out[idx]->type,var_prc_out[idx]->sz,var_prc[idx]->has_mss_val,var_prc[idx]->mss_val,var_prc[idx]->tally,var_prc_out[idx]->val,var_prc_out[idx]->val);
+        break;
+      case nco_op_sqravg: /* Square mean to create square of the mean (for sdn) */
+        (void)nco_var_mlt(var_prc_out[idx]->type,var_prc_out[idx]->sz,var_prc_out[idx]->has_mss_val,var_prc_out[idx]->mss_val,var_prc_out[idx]->val,var_prc_out[idx]->val);
+        break;
+      default:
+        break;
+      } /* end switch */
+    } /* !var_prc[idx]->is_crd_var */
+  } /* end (OpenMP parallel for) loop over variables */
+
+} /* end nco_opr_nrm() */
+
 void 
 nco_opr_drv /* [fnc] Intermediate control of arithmetic operations for ncra/ncea */
-(const long idx_rec, /* I [idx] Index of current record */
+(const long idx_rec, /* I [idx] Index of record (ncra) or file (ncea) in current operation group */
  const int nco_op_typ, /* I [enm] Operation type */
  const var_sct * const var_prc, /* I [sct] Variable in input file */
  var_sct * const var_prc_out) /* I/O [sct] Variable in output file */
@@ -43,13 +121,20 @@ nco_opr_drv /* [fnc] Intermediate control of arithmetic operations for ncra/ncea
   /* Purpose: Perform appropriate ncra/ncea operation (avg, min, max, ttl, ...) on operands
      nco_opr_drv() is called within the record loop of ncra, and within file loop of ncea
      These operations perform some, but not all, of necessary operations for each procedure
-     Most arithmetic operations require additional procedures such as normalization be performed after all files/records have been processed */
+     Most arithmetic operations require additional procedures such as normalization be performed after all files/records have been processed
+     Some operations require special care at initialization
+     This determination is based on the idx_rec variable
+     When idx_rec == 0, these operations may perform special initializations
+     The exact numeric value of idx_rec does not matter
+     What matters is whether it is zero or non-zero */
   
   /* NCO's paradigm is that coordinate variables represent grid axes
      Reducing such grids to a single-value must be done
      The most representative value of the grid is the average 
      The total, min, max, rms, etc. of the grid usually makes no sense
-     Users are most interested in the mean grid coordinate */
+     Users are most interested in the mean grid coordinate
+     20130112: The same logic applies to CF-style coordinates, e.g., 
+     to variables matching the CF "bounds" and "coordinates" conventions */
   if(var_prc->is_crd_var){
     (void)nco_var_add_tll_ncra(var_prc->type,var_prc->sz,var_prc->has_mss_val,var_prc->mss_val,var_prc->tally,var_prc->val,var_prc_out->val);
     return;
@@ -66,24 +151,24 @@ nco_opr_drv /* [fnc] Intermediate control of arithmetic operations for ncra/ncea
     /* On first loop, simply copy variables from var_prc to var_prc_out */
     if(idx_rec == 0) (void)nco_var_copy(var_prc->type,var_prc->sz,var_prc->val,var_prc_out->val); else (void)nco_var_max_bnr(var_prc_out->type,var_prc_out->sz,var_prc->has_mss_val,var_prc->mss_val,var_prc->val,var_prc_out->val);
     break;	
-  case nco_op_ttl: /* Total */ 
+  case nco_op_ttl: /* Total */
     /* NB: copying input to output on first loop for nco_op_ttl, in similar manner to nco_op_[max/min], can work
-       However, the copying with nco_var_copy() would not change the tally variable, leaving it equal to zero
+       However, copying with nco_var_copy() would not change the tally variable, leaving it equal to zero
        Then an extra step would be necessary to set tally equal to one where missing values were not present
        Otherwise, e.g., ensemble averages of one file would never have non-zero tallies
-       Hence, use special nco_var_copy_tll() function on to copy and change tally only in first loop iteration
+       Hence, use special nco_var_copy_tll() function to copy and change tally only in first loop iteration
        This way, tally is self-consistent with var_prc_out at all times
-       Other option is to use nco_var_add_tll_ncra() below and then to post-process nco_op_ttl with nco_var_tll_zro_mss_val()
-       in parent function (i.e., in ncra.c).
-       However, that method has downside that post-processing must be done in parent function
-       By using nco_var_copy_tll() in first iteration here, we avoid performing post-processing in parent function */
+       Moreover, running total must never be set to missing_value, because subsequent additions
+       (with nco_var_add_tll_ncra()) only check new addend (not running sum) against missing value.
+       Hence (as of 20120521) nco_var_copy_tll() specifically resets sum to zero rather than to missing value
+       Parent function (e.g., ncra.c) must post-process ttl buffers nco_op_ttl with nco_var_tll_zro_mss_val() */
     if(idx_rec == 0) (void)nco_var_copy_tll(var_prc->type,var_prc->sz,var_prc->has_mss_val,var_prc->mss_val,var_prc->tally,var_prc->val,var_prc_out->val); else (void)nco_var_add_tll_ncra(var_prc->type,var_prc->sz,var_prc->has_mss_val,var_prc->mss_val,var_prc->tally,var_prc->val,var_prc_out->val);
     break;
   case nco_op_avg: /* Average */
   case nco_op_sqrt: /* Squareroot will produce the squareroot of the mean */
   case nco_op_sqravg: /* Square of the mean */
     /* These operations all require subsequent normalization, where degenerate tallies are accounted for
-       Thus, the all call nco_var_add_tll_ncra() every iteration, without special treatment on first iteration */
+       Thus, they all call nco_var_add_tll_ncra() every iteration, without special treatment on first iteration */
     (void)nco_var_add_tll_ncra(var_prc->type,var_prc->sz,var_prc->has_mss_val,var_prc->mss_val,var_prc->tally,var_prc->val,var_prc_out->val);
     break;
   case nco_op_rms: /* Root mean square */
@@ -197,22 +282,22 @@ vec_set /* [fnc] Fill every value of first operand with value of second operand
     for(idx=0;idx<sz;idx++) op1.dp[idx]=op2;
     break;
   case NC_INT:
-    for(idx=0;idx<sz;idx++) op1.ip[idx]=(nco_int)lround(op2); /* Coerce to avoid C++ compiler assignment warning */
+    for(idx=0;idx<sz;idx++) op1.ip[idx]=(nco_int)lrint(op2); /* Coerce to avoid C++ compiler assignment warning */
     break;
   case NC_SHORT:
-    for(idx=0;idx<sz;idx++) op1.sp[idx]=(nco_short)lround(op2); /* Coerce to avoid C++ compiler assignment warning */
+    for(idx=0;idx<sz;idx++) op1.sp[idx]=(nco_short)lrint(op2); /* Coerce to avoid C++ compiler assignment warning */
     break;
   case NC_USHORT:
-    for(idx=0;idx<sz;idx++) op1.usp[idx]=(nco_ushort)lround(op2); /* Coerce to avoid C++ compiler assignment warning */
+    for(idx=0;idx<sz;idx++) op1.usp[idx]=(nco_ushort)lrint(op2); /* Coerce to avoid C++ compiler assignment warning */
     break;
   case NC_UINT:
-    for(idx=0;idx<sz;idx++) op1.uip[idx]=(nco_uint)lround(op2); /* Coerce to avoid C++ compiler assignment warning */
+    for(idx=0;idx<sz;idx++) op1.uip[idx]=(nco_uint)lrint(op2); /* Coerce to avoid C++ compiler assignment warning */
     break;
   case NC_INT64:
-    for(idx=0;idx<sz;idx++) op1.i64p[idx]=(nco_int64)llround(op2); /* Coerce to avoid C++ compiler assignment warning */
+    for(idx=0;idx<sz;idx++) op1.i64p[idx]=(nco_int64)llrint(op2); /* Coerce to avoid C++ compiler assignment warning */
     break;
   case NC_UINT64:
-    for(idx=0;idx<sz;idx++) op1.ui64p[idx]=(nco_uint64)llround(op2); /* Coerce to avoid C++ compiler assignment warning */
+    for(idx=0;idx<sz;idx++) op1.ui64p[idx]=(nco_uint64)llrint(op2); /* Coerce to avoid C++ compiler assignment warning */
     break;
   case NC_BYTE: break; /* Do nothing */
   case NC_UBYTE: break; /* Do nothing */
@@ -233,19 +318,16 @@ nco_zero_long /* [fnc] Zero all values of long array */
 {
   /* Purpose: Zero all values of long array */
 
-  size_t sz_byt; /* [B] Number of bytes in variable buffer */
-  sz_byt=(size_t)sz*sizeof(long);
-  (void)memset((void *)op1,(long)0,sz_byt);
-
-#if 0
-  /* Presumably this old method used until 20050321 is slower because of pointer de-referencing */
-  long idx;
+  /* Presumably this old method used until 20050321, and then again after 20120330,
+     is slower than memset() because of pointer de-referencing. 
+     However, it does have the virtue of being correct. */
   if(op1 == NULL){
     (void)fprintf(stdout,"%s: ERROR nco_zero_long() asked to zero NULL pointer\n",prg_nm_get());
     nco_exit(EXIT_FAILURE);
   } /* endif */
-  for(idx=0;idx<sz;idx++) op1[idx]=0L;
-#endif /* !0 */
+  size_t sz_byt; /* [B] Number of bytes in variable buffer */
+  sz_byt=(size_t)sz*sizeof(long);
+  (void)memset((void *)op1,0,sz_byt);
 
 } /* end nco_zero_long() */
 
@@ -257,9 +339,11 @@ nco_set_long /* [fnc] Set all values of long array */
 {
   /* Purpose: Set all values of long array to input value */
 
-  size_t sz_byt; /* [B] Number of bytes in variable buffer */
-  sz_byt=(size_t)sz*sizeof(long);
-  (void)memset((void *)op1,(long)val,sz_byt);
+  long idx;
+  if(op1 == NULL){
+    (void)fprintf(stdout,"%s: ERROR nco_set_long() asked to set NULL pointer\n",prg_nm_get());
+    nco_exit(EXIT_FAILURE);
+  } /* endif */
+  for(idx=0;idx<sz;idx++) op1[idx]=val;
 
 } /* end nco_set_long() */
-
