@@ -1,8 +1,8 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_var_lst.h,v 1.56 2012/02/14 01:10:05 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_var_lst.h,v 1.70 2013/06/14 02:56:14 pvicente Exp $ */
 
 /* Purpose: Variable list utilities */
 
-/* Copyright (C) 1995--2012 Charlie Zender
+/* Copyright (C) 1995--2013 Charlie Zender
    License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
@@ -17,9 +17,10 @@
 #endif /* !HAVE_CONFIG_H */
 
 /* Standard header files */
+#include <assert.h> /* assert() */
 #include <stdio.h> /* stderr, FILE, NULL, printf */
 #include <stdlib.h> /* strtod, strtol, malloc, getopt, exit */
-#include <string.h> /* strcmp. . . */
+#include <string.h> /* strcmp() */
 
 #ifdef HAVE_REGEX_H 
 /* 20120213: Linux GCC 4.6 man page says regex.h depends on (non-present) sys/types.h */
@@ -51,14 +52,14 @@ nco_var_lst_mk /* [fnc] Create variable extraction list */
  char * const * const var_lst_in, /* I [sng] User-specified list of variable names and rx's */
  const nco_bool EXCLUDE_INPUT_LIST, /* I [flg] Exclude rather than extract */
  const nco_bool EXTRACT_ALL_COORDINATES, /* I [flg] Process all coordinates */
- int * const var_nbr_xtr); /* I/O [nbr] Number of variables in current extraction list */
+ int * const var_xtr_nbr); /* I/O [nbr] Number of variables in current extraction list */
 
 nm_id_sct * /* O [sct] Extraction list */
 nco_var_lst_xcl /* [fnc] Convert exclusion list to extraction list */
 (const int nc_id, /* I netCDF file ID */
  const int nbr_var, /* I [nbr] Number of variables in input file */
  nm_id_sct *xtr_lst, /* I/O [sct] Current exclusion list (destroyed) */
- int * const nbr_xtr); /* I/O [nbr] Number of variables in exclusion/extraction list */
+ int * const xtr_nbr); /* I/O [nbr] Number of variables in exclusion/extraction list */
 
 nm_id_sct * /* O [sct] Extraction list */
 nco_var_lst_crd_add /* [fnc] Add all coordinates to extraction list */
@@ -66,24 +67,14 @@ nco_var_lst_crd_add /* [fnc] Add all coordinates to extraction list */
  const int nbr_dim, /* I [nbr] Number of dimensions in input file */
  const int nbr_var, /* I [nbr] Number of variables in input file */
  nm_id_sct *xtr_lst, /* I/O [sct] Current extraction list (destroyed) */
- int * const nbr_xtr, /* I/O [nbr] Number of variables in current extraction list */
+ int * const xtr_nbr, /* I/O [nbr] Number of variables in current extraction list */
  const nco_bool CNV_CCM_CCSM_CF); /* I [flg] file obeys CCM/CCSM/CF conventions */
 
-nco_bool /* [flg] Variable is listed in a "bounds" attribute */
-nco_is_spc_in_bnd_att /* [fnc] Variable is listed in a "bounds" attribute */
-(const int nc_id, /* I [id] netCDF file ID */
- const int var_trg_id); /* I [id] Variable ID */
-
-nco_bool /* [flg] Variable is listed in a "coordinates" attribute */
-nco_is_spc_in_crd_att /* [fnc] Variable is listed in a "coordinates" attribute */
-(const int nc_id, /* I [id] netCDF file ID */
- const int var_trg_id); /* I [id] Variable ID */
-
 nm_id_sct * /* O [sct] Extraction list */
 nco_var_lst_crd_ass_add /* [fnc] Add to extraction list all coordinates associated with extracted variables */
 (const int nc_id, /* I netCDF file ID */
  nm_id_sct *xtr_lst, /* I/O current extraction list (destroyed) */
- int * const nbr_xtr, /* I/O number of variables in current extraction list */
+ int * const xtr_nbr, /* I/O number of variables in current extraction list */
  const nco_bool CNV_CCM_CCSM_CF); /* I [flg] file obeys CCM/CCSM/CF conventions */
 
 nm_id_sct * /* O [sct] List with coordinate excluded */
@@ -91,13 +82,23 @@ nco_var_lst_crd_xcl /* [fnc] Exclude given coordinates from extraction list */
 (const int nc_id, /* I [id] netCDF file ID */
  const int dmn_id, /* I [id] Dimension ID of coordinate to remove from extraction list */
  nm_id_sct *xtr_lst, /* I/O [sct] Current extraction list (destroyed) */
- int * const nbr_xtr); /* I/O [nbr] Number of variables in extraction list */
+ int * const xtr_nbr); /* I/O [nbr] Number of variables in extraction list */
+
+void
+nco_var_lst_fix_rec_dvd /* [fnc] Divide extraction list into fixed and record data */
+(const int nc_id, /* I [id] netCDF file ID */
+ nm_id_sct *xtr_lst, /* I/O [sct] Extraction list (pointers to it are constructed) */
+ const int xtr_nbr, /* I [nbr] Number of variables in extraction list */
+ nm_id_sct ***fix_lst, /* O [sct] Fixed-length variables */
+ int * const fix_nbr, /* O [nbr] Number of fixed-length variables */
+ nm_id_sct ***rec_lst, /* O [sct] Record variables */
+ int * const rec_nbr); /* O [nbr] Number of record variables */
 
 void
 nco_var_lst_convert /* [fnc] Make variable structure list from variable name ID list */
 (const int nc_id, /* I [enm] netCDF file ID */
  nm_id_sct *xtr_lst, /* I [sct] Current extraction list (destroyed) */
- const int nbr_xtr, /* I [nbr] Number of variables in input file */
+ const int xtr_nbr, /* I [nbr] Number of variables in input file */
  dmn_sct * const * const dim, /* I [sct] Dimensions associated with input variable list */
  const int nbr_dmn_xtr, /* I [nbr] Number of dimensions in list  */
  var_sct *** const var_ptr, /* O [sct] Variable list (for input file) */
@@ -109,6 +110,7 @@ nco_var_lst_dvd /* [fnc] Divide input lists into output lists */
  var_sct * const * const var_out, /* I [sct] Variable list (output file) */
  const int nbr_var, /* I [nbr] Number of variables */
  const nco_bool CNV_CCM_CCSM_CF, /* I [flg] File adheres to NCAR CCM/CCSM/CF conventions */
+ const nco_bool FIX_REC_CRD, /* [flg] Do not interpolate/multiply record coordinate variables (ncflint only) */
  const int nco_pck_map, /* I [enm] Packing map */
  const int nco_pck_plc, /* I [enm] Packing policy */
  CST_X_PTR_CST_PTR_CST_Y(dmn_sct,dmn_xcl), /* I [sct] Dimensions not allowed in fixed variables */
@@ -127,6 +129,22 @@ nco_var_lst_mrg /* [fnc] Merge two variable lists into same order */
  int * const var_nbr_1, /* I/O [nbr] Number of variables in list 1 */
  int * const var_nbr_2); /* I/O [nbr] Number of variables in list 2 */
 
+void
+nco_var_lst_dvd_trv                          /* [fnc] Divide input lists into output lists (ncbo only) */
+(var_sct * const var,                        /* I [sct] Variable list (input file) */
+ var_sct * const var_out,                    /* I [sct] Variable list (output file) */
+ const nco_bool CNV_CCM_CCSM_CF,             /* I [flg] File adheres to NCAR CCM/CCSM/CF conventions */
+ const nco_bool FIX_REC_CRD,                 /* I [flg] Do not interpolate/multiply record coordinate variables (ncflint only) */
+ const int nco_pck_map,                      /* I [enm] Packing map */
+ const int nco_pck_plc,                      /* I [enm] Packing policy */
+ CST_X_PTR_CST_PTR_CST_Y(dmn_sct,dmn_xcl),   /* I [sct] Dimensions not allowed in fixed variables */
+ const int nbr_dmn_xcl,                      /* I [nbr] Number of altered dimensions */
+ prc_typ_enm *prc);                          /* O [enm] Processing type */
+
+
+
+
+
 #ifdef __cplusplus
 } /* end extern "C" */
 #endif /* __cplusplus */
