@@ -1,8 +1,8 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_var_rth.c,v 1.59 2012/02/07 07:12:05 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_var_rth.c,v 1.64 2013/01/13 06:07:48 zender Exp $ */
 
 /* Purpose: Variable arithmetic */
 
-/* Copyright (C) 1995--2012 Charlie Zender
+/* Copyright (C) 1995--2013 Charlie Zender
    License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
@@ -550,7 +550,6 @@ nco_var_add_tll_ncra /* [fnc] Add first operand to second operand, increment tal
   /* Purpose: Add value of first operand to value of second operand 
      and store result in second operand. 
      Assume operands conform, are same type, and are in memory
-     nco_var_add() adds to tally if neither operand equals missing_value
      nco_var_add() does _not_ increment tally counter.
      nco_var_add_tll_ncflint() adds if neither operand equals missing value
      nco_var_add_tll_ncflint() does increment tally counter (unlike nco_var_add())
@@ -726,7 +725,7 @@ nco_var_add_tll_ncra /* [fnc] Add first operand to second operand, increment tal
 } /* end nco_var_add_tll_ncra() */
 
 void 
-nco_var_copy_tll /* [fnc] Copy hyperslab variables of type var_typ from op1 to op2 */
+nco_var_copy_tll /* [fnc] Copy hyperslab variables of type var_typ from op1 to op2, accounting for missing values in tally */
 (const nc_type type, /* I [enm] netCDF type */
  const long sz, /* I [nbr] Number of elements to copy */
  const int has_mss_val, /* I [flg] Flag for missing values */
@@ -748,10 +747,14 @@ nco_var_copy_tll /* [fnc] Copy hyperslab variables of type var_typ from op1 to o
   /* Use fast nco_var_copy() method to copy variable */
   (void)memcpy((void *)(op2.vp),(void *)(op1.vp),sz*nco_typ_lng(type));
 
-  /* Tally is one if no missing value is defined */
-  (void)nco_set_long(sz,1L,tally);
-
-  if(!has_mss_val) return;
+  if(has_mss_val){
+    /* Tally remains zero until verified (below) that datum is not missing value */
+    (void)nco_set_long(sz,0L,tally);
+  }else{ /* !has_mss_val */
+    /* Tally is one if no missing value is defined */
+    (void)nco_set_long(sz,1L,tally);
+    return;
+  } /* !has_mss_val */
 
   /* Typecast pointer to values before access */
   (void)cast_void_nctype(type,&op2);
@@ -762,49 +765,49 @@ nco_var_copy_tll /* [fnc] Copy hyperslab variables of type var_typ from op1 to o
   case NC_FLOAT:
     {
       const float mss_val_flt=*mss_val.fp;
-      for(idx=0;idx<sz;idx++) if(op2.fp[idx] == mss_val_flt) tally[idx]=0L;
+      for(idx=0;idx<sz;idx++) if(op2.fp[idx] == mss_val_flt) op2.fp[idx]=0.0f; else tally[idx]=1L;
     }
     break;
   case NC_DOUBLE:
     {
       const double mss_val_dbl=*mss_val.dp;
-      for(idx=0;idx<sz;idx++) if(op2.dp[idx] == mss_val_dbl) tally[idx]=0L;
+      for(idx=0;idx<sz;idx++) if(op2.dp[idx] == mss_val_dbl) op2.dp[idx]=0.0; else tally[idx]=1L;
     }
     break;
   case NC_INT:
     {
       const nco_int mss_val_ntg=*mss_val.ip;
-      for(idx=0;idx<sz;idx++) if(op2.ip[idx] == mss_val_ntg) tally[idx]=0L;
+      for(idx=0;idx<sz;idx++) if(op2.ip[idx] == mss_val_ntg) op2.ip[idx]=0; else tally[idx]=1L;
     }
     break;
   case NC_SHORT:
     {
       const nco_short mss_val_short=*mss_val.sp;
-      for(idx=0;idx<sz;idx++) if(op2.sp[idx] == mss_val_short) tally[idx]=0L;
+      for(idx=0;idx<sz;idx++) if(op2.sp[idx] == mss_val_short) op2.sp[idx]=0; else tally[idx]=1L;
     }
     break;
   case NC_USHORT:
     {
       const nco_ushort mss_val_ushort=*mss_val.usp;
-      for(idx=0;idx<sz;idx++) if(op2.usp[idx] == mss_val_ushort) tally[idx]=0L;
+      for(idx=0;idx<sz;idx++) if(op2.usp[idx] == mss_val_ushort) op2.usp[idx]=0; else tally[idx]=1L;
     }
     break;
   case NC_UINT:
     {
       const nco_uint mss_val_uint=*mss_val.uip;
-      for(idx=0;idx<sz;idx++) if(op2.uip[idx] == mss_val_uint) tally[idx]=0L;
+      for(idx=0;idx<sz;idx++) if(op2.uip[idx] == mss_val_uint) op2.uip[idx]=0; else tally[idx]=1L;
     }
     break;
   case NC_INT64:
     {
       const nco_int64 mss_val_int64=*mss_val.i64p;
-      for(idx=0;idx<sz;idx++) if(op2.i64p[idx] == mss_val_int64) tally[idx]=0L;
+      for(idx=0;idx<sz;idx++) if(op2.i64p[idx] == mss_val_int64) op2.i64p[idx]=0; else tally[idx]=1L;
     }
     break;
   case NC_UINT64:
     {
       const nco_uint64 mss_val_uint64=*mss_val.ui64p;
-      for(idx=0;idx<sz;idx++) if(op2.ui64p[idx] == mss_val_uint64) tally[idx]=0L;
+      for(idx=0;idx<sz;idx++) if(op2.ui64p[idx] == mss_val_uint64) op2.ui64p[idx]=0; else tally[idx]=1L;
     }
     break;
   case NC_BYTE: break; /* Do nothing */
@@ -1653,7 +1656,7 @@ nco_var_tll_zro_mss_val /* [fnc] Write missing value into elements with zero tal
      Such sums are usually normalized and set to missing if tally is zero.
      However, totals are integrals and thus are never normalized.
      Initialization value of zero will be output even if tally is zero,
-     _unless field is processed with this routine after summing and prior to writing. */
+     _unless field is processed with this routine after summing and prior to writing_ */
   
   /* Filter currently works as op1:=mss_val where tally == 0 */
   
