@@ -1,4 +1,4 @@
-/* $Header: /cvsroot/nco/nco/src/nco/ncwa.c,v 1.370 2013/09/20 20:58:19 pvicente Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/ncwa.c,v 1.376 2013/09/27 22:21:37 pvicente Exp $ */
 
 /* ncwa -- netCDF weighted averager */
 
@@ -133,8 +133,8 @@ main(int argc,char **argv)
   char *wgt_nm=NULL;
   char trv_pth[]="/"; /* [sng] Root path of traversal tree */
 
-  const char * const CVS_Id="$Id: ncwa.c,v 1.370 2013/09/20 20:58:19 pvicente Exp $"; 
-  const char * const CVS_Revision="$Revision: 1.370 $";
+  const char * const CVS_Id="$Id: ncwa.c,v 1.376 2013/09/27 22:21:37 pvicente Exp $"; 
+  const char * const CVS_Revision="$Revision: 1.376 $";
   const char * const opt_sht_lst="346Aa:B:bCcD:d:Fg:G:hIL:l:M:m:nNOo:p:rRT:t:v:Ww:xy:-:";
 
   cnk_sct **cnk=NULL_CEWI;
@@ -176,7 +176,6 @@ main(int argc,char **argv)
   int fll_md_old; /* [enm] Old fill mode */
   int grp_lst_in_nbr=0; /* [nbr] Number of groups explicitly specified by user */
   int idx=int_CEWI;
-  int idx_avg;
   int in_id;  
   int lmt_nbr=0; /* Option d. NB: lmt_nbr gets incremented */
   int md_open; /* [enm] Mode flag for nc_open() call */
@@ -196,12 +195,8 @@ main(int argc,char **argv)
   int thr_nbr=int_CEWI; /* [nbr] Thread number Option t */
   int var_lst_in_nbr=0;
 
-  lmt_sct **lmt; 
-
   md5_sct *md5=NULL; /* [sct] MD5 configuration */
 
-  nm_id_sct *dmn_avg_lst;
-
   size_t bfr_sz_hnt=NC_SIZEHINT_DEFAULT; /* [B] Buffer size hint */
   size_t cnk_sz_scl=0UL; /* [nbr] Chunk size scalar */
   size_t hdr_pad=0UL; /* [B] Pad at end of header section */
@@ -212,11 +207,7 @@ main(int argc,char **argv)
   var_sct **var_out;
   var_sct **var_prc;
   var_sct **var_prc_out;
-  var_sct *msk=NULL;
-  var_sct *msk_out=NULL;
-  var_sct *wgt=NULL;
   var_sct *wgt_avg=NULL;
-  var_sct *wgt_out=NULL;
 
   trv_tbl_sct *trv_tbl=NULL; /* [lst] Traversal table */
 
@@ -600,9 +591,6 @@ main(int argc,char **argv)
   /* Make uniform list of user-specified chunksizes */
   if(cnk_nbr > 0) cnk=nco_cnk_prs(cnk_nbr,cnk_arg);
 
-  /* Make uniform list of user-specified dimension limits */
-  lmt=nco_lmt_prs(lmt_nbr,lmt_arg);
-
   /* Initialize thread information */
   thr_nbr=nco_openmp_ini(thr_nbr);
   in_id_arr=(int *)nco_malloc(thr_nbr*sizeof(int));
@@ -618,7 +606,7 @@ main(int argc,char **argv)
   (void)nco_inq_format(in_id,&fl_in_fmt);
 
   /* Construct GTT, Group Traversal Table (groups,variables,dimensions, limits) */
-  (void)nco_bld_trv_tbl(in_id,trv_pth,MSA_USR_RDR,lmt_nbr,lmt,FORTRAN_IDX_CNV,aux_nbr,aux_arg,grp_lst_in,grp_lst_in_nbr,var_lst_in,xtr_nbr,EXTRACT_ALL_COORDINATES,GRP_VAR_UNN,EXCLUDE_INPUT_LIST,EXTRACT_ASSOCIATED_COORDINATES,trv_tbl);
+  (void)nco_bld_trv_tbl(in_id,trv_pth,lmt_nbr,lmt_arg,aux_nbr,aux_arg,MSA_USR_RDR,FORTRAN_IDX_CNV,grp_lst_in,grp_lst_in_nbr,var_lst_in,xtr_nbr,EXTRACT_ALL_COORDINATES,GRP_VAR_UNN,EXCLUDE_INPUT_LIST,EXTRACT_ASSOCIATED_COORDINATES,trv_tbl);
 
   /* Get number of variables, dimensions, and global attributes in file, file format */
   (void)trv_tbl_inq((int *)NULL,(int *)NULL,(int *)NULL,&nbr_dmn_fl,(int *)NULL,(int *)NULL,(int *)NULL,(int *)NULL,&nbr_var_fl,trv_tbl);
@@ -639,8 +627,6 @@ main(int argc,char **argv)
     if(dbg_lvl >= nco_dbg_std) (void)fprintf(stderr,"%s: INFO No dimensions specified with -a, therefore reducing (averaging, taking minimum, etc.) over all dimensions\n",prg_nm);
   } /* end if dmn_avg_nbr == 0 */
 
-
-
   /* Allocate array of dimensions to average with maximum possible size */
   dmn_avg=(dmn_sct **)nco_malloc(nbr_dmn_fl*sizeof(dmn_sct *));
 
@@ -662,23 +648,6 @@ main(int argc,char **argv)
   /* Fill-in variable structure list for all extracted variables. NOTE: Using GTT version */
   var=nco_fll_var_trv(in_id,&xtr_nbr,trv_tbl);
 
-  /* Merge hyperslab limit information into dimension structures */
-  if(lmt_nbr > 0) (void)nco_dmn_lmt_mrg(dim,nbr_dmn_xtr,lmt,lmt_nbr);
-
-  /* Update variable array with limits */
-  for(int idx_var=0;idx_var<xtr_nbr;idx_var++){
-    for(int idx_dmn_var=0;idx_dmn_var<var[idx_var]->nbr_dim;idx_dmn_var++){
-      for(int idx_dmn_xtr=0;idx_dmn_xtr<nbr_dmn_xtr;idx_dmn_xtr++){
-        if (var[idx_var]->dmn_id[idx_dmn_var] == dim[idx_dmn_xtr]->id){
-          var[idx_var]->srt[idx_dmn_var]=dim[idx_dmn_xtr]->srt;
-          var[idx_var]->srd[idx_dmn_var]=dim[idx_dmn_xtr]->srd;
-          var[idx_var]->end[idx_dmn_var]=dim[idx_dmn_xtr]->end;
-          var[idx_var]->cnt[idx_dmn_var]=dim[idx_dmn_xtr]->cnt;
-        }
-      }
-    }  
-  }
-
   /* Duplicate to output array */
   var_out=(var_sct **)nco_malloc(xtr_nbr*sizeof(var_sct *));
   for(idx=0;idx<xtr_nbr;idx++){
@@ -777,7 +746,6 @@ main(int argc,char **argv)
   /* Close first input netCDF file */
   nco_close(in_id);
 
-
   /* Loop over input files (not currently used, fl_nbr == 1) */
   for(fl_idx=0;fl_idx<fl_nbr;fl_idx++){
     /* Parse filename */
@@ -795,29 +763,6 @@ main(int argc,char **argv)
     /* Perform various error-checks on input file */
     if(False) (void)nco_fl_cmp_err_chk();
 
-    /* Find weighting variable in input file */
-    if(wgt_nm){
-
-      /* Retrieve weighting variable */
-      /* fxm: TODO #111 core dump if msk has dimension not in extraction list */
-      wgt=nco_var_get_trv(in_id,wgt_nm,trv_tbl);
-
-      /* fxm: Perhaps should allocate default tally array for wgt here
-      That way, when wgt conforms to the first var_prc_out and it therefore
-      does not get a tally array copied by nco_var_dpl() in nco_var_cnf_dmn(), 
-      it will at least have space for a tally array. TODO #114. */
-
-    } /* Find weighting variable in input file */
-
-    /* Find mask variable in input file */
-    if(msk_nm){
-
-      /* Retrieve mask variable */
-      /* fxm: TODO #111 core dump if msk has dimension not in extraction list */
-      msk=nco_var_get_trv(in_id,msk_nm,trv_tbl);
-
-    } /* Find mask variable in input file */
-
     /* Timestamp end of metadata setup and disk layout */
     rcd+=nco_ddra((char *)NULL,(char *)NULL,&ddra_info);
     ddra_info.tmr_flg=nco_tmr_rgl;
@@ -827,7 +772,7 @@ main(int argc,char **argv)
     firstprivate(): msk_out and wgt_out must be NULL on first call to nco_var_cnf_dmn()
     shared(): msk and wgt are not altered within loop
     private(): wgt_avg does not need initialization */
-#pragma omp parallel for default(none) firstprivate(DO_CONFORM_MSK,DO_CONFORM_WGT,ddra_info,msk_out,wgt_out) private(idx,in_id,wgt_avg) shared(MULTIPLY_BY_TALLY,MUST_CONFORM,NRM_BY_DNM,WGT_MSK_CRD_VAR,dbg_lvl,dmn_avg,dmn_avg_nbr,flg_ddra,flg_rdd,gpe,in_id_arr,msk,msk_nm,msk_val,nbr_var_prc,nco_op_typ,op_typ_rlt,out_id,prg_nm,rcd,trv_tbl,var_prc,var_prc_out,wgt,wgt_nm)
+#pragma omp parallel for default(none) firstprivate(DO_CONFORM_MSK,DO_CONFORM_WGT,ddra_info) private(idx,in_id,wgt_avg) shared(MULTIPLY_BY_TALLY,MUST_CONFORM,NRM_BY_DNM,WGT_MSK_CRD_VAR,dbg_lvl,dmn_avg,dmn_avg_nbr,flg_ddra,flg_rdd,gpe,in_id_arr,msk_nm,msk_val,nbr_var_prc,nco_op_typ,op_typ_rlt,out_id,prg_nm,rcd,trv_tbl,var_prc,var_prc_out,wgt_nm)
 #endif /* !_OPENMP */
 
     for(idx=0;idx<nbr_var_prc;idx++){ /* Process all variables in current file */
@@ -837,11 +782,33 @@ main(int argc,char **argv)
       int grp_id;        /* [ID] Group ID */
       int grp_out_id;    /* [ID] Group ID (output) */
       int var_out_id;    /* [ID] Variable ID (output) */
+
       trv_sct *var_trv;  /* [sct] Variable GTT object */
 
+      var_sct *wgt=NULL;
+      var_sct *wgt_out=NULL;
+      var_sct *msk=NULL;
+      var_sct *msk_out=NULL;
 
       in_id=in_id_arr[omp_get_thread_num()];
 
+      /* Find weighting variable that matches current variable */
+      if(wgt_nm){
+
+        /* Retrieve weighting variable */
+        wgt=nco_var_get_wgt_trv(in_id,wgt_nm,var_prc[idx],trv_tbl);
+
+      } /* Find weighting variable that matches current variable */
+
+
+      /* Find mask variable that matches current variable */
+      if(msk_nm){
+
+        /* Retrieve mask variable */
+        msk=nco_var_get_wgt_trv(in_id,msk_nm,var_prc[idx],trv_tbl);
+
+      } /* Find mask variable that matches current variable */   
+
       /* Obtain variable GTT object using full variable name */
       var_trv=trv_tbl_var_nm_fll(var_prc[idx]->nm_fll,trv_tbl);
 
@@ -871,13 +838,12 @@ main(int argc,char **argv)
       /* Retrieve variable from disk into memory */
       (void)nco_msa_var_get_trv(in_id,var_prc[idx],trv_tbl);
 
-      if(False) (void)fprintf(fp_stdout,"%s: DEBUG: fxm TODO nco354 Finished nco_var_get() %s\n",prg_nm,var_prc[idx]->nm);
-
       /* Convert char, short, long, int types to doubles before arithmetic */
       var_prc[idx]=nco_typ_cnv_rth(var_prc[idx],nco_op_typ);
       var_prc_out[idx]=nco_typ_cnv_rth(var_prc_out[idx],nco_op_typ);
 
-      if(msk_nm && (!var_prc[idx]->is_crd_var || WGT_MSK_CRD_VAR)){
+      /* Check mask found for this variable, using msk */
+      if(msk && (!var_prc[idx]->is_crd_var || WGT_MSK_CRD_VAR)){
         msk_out=nco_var_cnf_dmn(var_prc[idx],msk,msk_out,MUST_CONFORM,&DO_CONFORM_MSK);
         /* If msk and var did not conform then do not mask var! */
         if(DO_CONFORM_MSK){
@@ -905,7 +871,9 @@ main(int argc,char **argv)
           break;
         } /* end case */
       } /* var_prc[idx]->is_crd_var */
-      if(wgt_nm && (!var_prc[idx]->is_crd_var || WGT_MSK_CRD_VAR)){
+
+      /* Check weight found for this variable, using wgt */
+      if(wgt && (!var_prc[idx]->is_crd_var || WGT_MSK_CRD_VAR)){
         /* fxm: nco_var_cnf_dmn() has bug where it does not allocate tally array
         for weights that do already conform to var_prc. TODO #114. */
         wgt_out=nco_var_cnf_dmn(var_prc[idx],wgt,wgt_out,MUST_CONFORM,&DO_CONFORM_WGT);
@@ -962,7 +930,7 @@ main(int argc,char **argv)
         /* Free current input buffer */
         var_prc[idx]->val.vp=nco_free(var_prc[idx]->val.vp);
 
-        if(msk_nm && DO_CONFORM_MSK){
+        if(msk && DO_CONFORM_MSK){
           /* Must mask weight in same fashion as variable was masked
           If msk and var did not conform then do not mask wgt
           Ensure wgt_avg has a missing value */
@@ -1122,6 +1090,12 @@ main(int argc,char **argv)
       /* Free current output buffer */
       var_prc_out[idx]->val.vp=nco_free(var_prc_out[idx]->val.vp);
 
+      /* Free possible weight/mask found */
+      if(wgt) wgt=nco_var_free(wgt);
+      if(wgt_out) wgt_out=nco_var_free(wgt_out);
+      if(msk) msk=nco_var_free(msk);
+      if(msk_out) msk_out=nco_var_free(msk_out);
+
     } /* end (OpenMP parallel for) loop over idx */
 
     if(dbg_lvl >= nco_dbg_var) (void)fprintf(stderr,"\n");
@@ -1137,24 +1111,14 @@ main(int argc,char **argv)
   /* Close output file and move it from temporary to permanent location */
   (void)nco_fl_out_cls(fl_out,fl_out_tmp,out_id);
 
-
-
   /* Clean memory unless dirty memory allowed */
   if(flg_cln){
     /* ncwa-specific memory */
     if(dmn_avg_nbr > 0) dmn_avg=(dmn_sct **)nco_free(dmn_avg);
-    if(msk) msk=nco_var_free(msk);
     if(msk_nm) msk_nm=(char *)nco_free(msk_nm);
-    if(msk_out) msk_out=nco_var_free(msk_out);
     if(msk_cnd_sng) msk_cnd_sng=(char *)nco_free(msk_cnd_sng);
-    if(wgt) wgt=nco_var_free(wgt);
     if(wgt_avg) wgt_avg=nco_var_free(wgt_avg);
     if(wgt_nm) wgt_nm=(char *)nco_free(wgt_nm);
-    if(wgt_out) wgt_out=nco_var_free(wgt_out);
-
-    /* Free lmt */
-    if(lmt_nbr > 0) lmt=nco_lmt_lst_free(lmt,lmt_nbr);
-
     /* NCO-generic clean-up */
     /* Free individual strings/arrays */
     if(cmd_ln) cmd_ln=(char *)nco_free(cmd_ln);
@@ -1171,7 +1135,6 @@ main(int argc,char **argv)
     if(fl_lst_in && fl_lst_abb) fl_lst_in=nco_sng_lst_free(fl_lst_in,1);
     if(fl_lst_abb) fl_lst_abb=nco_sng_lst_free(fl_lst_abb,abb_arg_nbr);
     if(var_lst_in_nbr > 0) var_lst_in=nco_sng_lst_free(var_lst_in,var_lst_in_nbr);
-
     /* Free chunking information */
     for(idx=0;idx<cnk_nbr;idx++) cnk_arg[idx]=(char *)nco_free(cnk_arg[idx]);
     if(cnk_nbr > 0) cnk=nco_cnk_lst_free(cnk,cnk_nbr);
