@@ -1,8 +1,8 @@
-// $Header: /cvsroot/nco/nco/src/nco_c++/tst.cc,v 1.35 2012/01/01 20:51:54 zender Exp $
+// $Header: /cvsroot/nco/nco/src/nco_c++/tst.cc,v 1.37 2013/01/13 06:07:49 zender Exp $
 
 // Purpose: Test for C++ programs using libnco_c++ API
 
-/* Copyright (C) 2001--2012 Charlie Zender
+/* Copyright (C) 2001--2013 Charlie Zender
    License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
@@ -58,10 +58,10 @@ int main(int argc,char **argv)
   long sz_nbr(1); // [nbr] Number of sizes
   register long idx; // [idx] Counting index
 
-  const std::string CVS_Date("$Date: 2012/01/01 20:51:54 $"); // [sng] CVS date string
-  const std::string CVS_Header("$Header: /cvsroot/nco/nco/src/nco_c++/tst.cc,v 1.35 2012/01/01 20:51:54 zender Exp $"); // [sng] CVS header string
-  const std::string CVS_Id("$Id: tst.cc,v 1.35 2012/01/01 20:51:54 zender Exp $"); // [sng] CVS identification string
-  const std::string CVS_Revision("$Revision: 1.35 $"); // [sng] CVS revision string
+  const std::string CVS_Date("$Date: 2013/01/13 06:07:49 $"); // [sng] CVS date string
+  const std::string CVS_Header("$Header: /cvsroot/nco/nco/src/nco_c++/tst.cc,v 1.37 2013/01/13 06:07:49 zender Exp $"); // [sng] CVS header string
+  const std::string CVS_Id("$Id: tst.cc,v 1.37 2013/01/13 06:07:49 zender Exp $"); // [sng] CVS identification string
+  const std::string CVS_Revision("$Revision: 1.37 $"); // [sng] CVS revision string
   const std::string date_cvs(CVS_Date.length() > 7 ? CVS_Date.substr(7,19) : static_cast<std::string>("Unknown")); // [sng] Date from CVS
   const std::string sbr_nm("main"); // [sng] Subroutine name
   const std::string prg_nm("libnco_c++"); // [sng] Program name
@@ -107,20 +107,20 @@ int main(int argc,char **argv)
 # define NC_FORMAT_CLASSIC (1)
 #endif
   int fl_out_fmt(NC_FORMAT_CLASSIC); // [enm] Output file format
-  int nccreate_mode(NC_CLOBBER); // [enm] Mode flag for nco_create() call
+  int md_create(NC_CLOBBER); // [enm] Mode flag for nco_create() call
 #ifdef ENABLE_NETCDF4
   fl_out_fmt=NC_FORMAT_NETCDF4; // [enm] Output file format
   if(fl_out_fmt == NC_FORMAT_64BIT){
-    nccreate_mode|=NC_64BIT_OFFSET;
+    md_create|=NC_64BIT_OFFSET;
   }else if(fl_out_fmt == NC_FORMAT_NETCDF4){
-    nccreate_mode|=NC_NETCDF4;
+    md_create|=NC_NETCDF4;
   }else if(fl_out_fmt == NC_FORMAT_NETCDF4_CLASSIC){
-    nccreate_mode|=NC_NETCDF4|NC_CLASSIC_MODEL;
+    md_create|=NC_NETCDF4|NC_CLASSIC_MODEL;
   } /* end else fl_out_fmt */
 #else // !ENABLE_NETCDF4
-  if(fl_out_fmt == NC_FORMAT_CLASSIC) nccreate_mode+=0; // CEWI
+  if(fl_out_fmt == NC_FORMAT_CLASSIC) md_create+=0; // CEWI
 #endif // !ENABLE_NETCDF4
-  const int nc_out(nco_create(fl_out,nccreate_mode)); 
+  const int nc_out(nco_create(fl_out,md_create)); 
   const nc_type nco_xtyp(nco_get_xtype(static_cast<prc_cmp>(1.0))); // [enm] External netCDF type
   std::cout << "INFO External netCDF type of prc_cmp variables will be " << nco_typ_sng(nco_xtyp) << std::endl;
  
